﻿using KSProgram;
using KSProgram.Data;
using KSProgram.Dokument;
using KSProgram.Extensions;
using KSProgram.Obecne;
using KSProgram.WebControls;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Telerik.Web.UI;

namespace KSPortal.Bulletin
{
    public partial class BulletinControl : System.Web.UI.UserControl
    {

        public bool Editacni
        {
            get
            {
                object o = ViewState["BC_Editacni"];
                return (bool)(o ?? false);
            }
            set
            {
                ViewState["BC_Editacni"] = value;
                RE_editor.Enabled = Editacni;
                RW_editor.VisibleStatusbar = Editacni;
                //P_command.Visible = Editacni;
            }
        }

        public int? IdBull
        {
            get
            {
                object o = ViewState["BC_IdBull"];
                return (int?)o;
            }
            set
            {
                ViewState["BC_IdBull"] = value;
            }
        }

        public bool Opened
        {
            get
            {
                return HF_opened.Value.ToLower() == "true";
            }
            set
            {
                HF_opened.Value = value.ToString();
            }
        }

        protected void Page_Load(object sender, EventArgs e)
        {
            //pridani overzovaných scriptu a stylu
            TRegisteredScripts.AddCssStyleToPage(this.Page, "~/Bulletin/BullEditor.css");
            TRegisteredScripts.AddCssStyleToPage(this.Page, "~/kendo/styles/kendo.common.min.css");
            TRegisteredScripts.AddCssStyleToPage(this.Page, "~/Skins/kendoMetro/kendoMetro.css");
            //registrace javascriptu
            TRegisteredScripts.AddScriptToPage(this.Page, "~/Bulletin/BullEditor.js", "BullControl");
            TRegisteredScripts.AddScriptToPage(this.Page, "~/kendo/js/kendo.all.min.js", "kendo");

            if (!IsPostBack)
            {
                creatreDirStructure();
                if (IdBull.HasValue)
                {
                    LoadBulletin(IdBull.Value);
                }
            }

            RW_editor.VisibleOnPageLoad = Opened;

            // select the Home tab by default
            if (RE_editor.RibbonBar != null)
                RE_editor.RibbonBar.SelectedTabIndex = 0;
            //handle postbacks from the export commands
            try
            {
                string evtArg = Request["__EVENTARGUMENT"];
                switch (evtArg)
                {
                    case "ksSave":
                        SaveBulletin();
                        break;
                    case "SaveAsDocx":
                        RE_editor.ExportToDocx();
                        break;
                    case "SaveAsRtf":
                        RE_editor.ExportToRtf();
                        break;
                    case "SaveAsPDF":
                        RE_editor.ExportToPdf();
                        break;
                    case "SaveAsMarkdown":
                        RE_editor.ExportToMarkdown();
                        break;
                    case "EditorClose":
                        CloseWindow();
                        break;
                    default:
                        break;
                }
            }
            catch (Exception ex)
            {
                RadNotification1.Show("There was an error during the export operation. Try simplifying the content and removing images/lists.");
            }
        }

        //protected override void OnPreRender(EventArgs e)
        //{
        //    base.OnPreRender(e);

        //    RSB_Editor.Visible = RW_editor.VisibleOnPageLoad;
        //}

        public void LoadBulletin(int id)
        {
            KSProgram.Bulletin.Bulletin bull = KSProgram.Bulletin.Bulletin.GetBulletins(id, null, null, null, true).FirstOrDefault();
            byte[] attachment = ((byte[])CacheInfo.GetItemFromCache("BullAttach_" + bull.id))?.UnZip();
            string html = bull.LoadHtml();
            HF_id.Value = id.ToString();
            if (attachment == null)
            {
                attachment = bull.LoadHtmlByte();

                //try
                //{
                //    PdfReader pdfreader = new PdfReader(attachment);
                //}
                //catch
                //{
                //    try
                //    {
                //        byte[] pdf = Convert.FromBase64String(html);
                //        PdfReader reader = new PdfReader(pdf);
                //        attachment = pdf;
                //    }
                //    catch
                //    {
                //        attachment = null;
                //    }

                //}
                byte[] pdf;
                if (PDF.IsValidPdf(attachment))
                {
                    //attachment je pdf
                }
                else if (html.TryParseBase64(out pdf) && PDF.IsValidPdf(pdf))
                {
                    attachment = pdf;
                }
                else
                {
                    attachment = null;
                }
            }
            if (attachment != null)
            {
                //PdfViewer1.Visible = true;
                //Btn_kontrola.Visible = true;
                //PdfViewer1.PdfjsProcessingSettings.FileSettings.Data = Convert.ToBase64String(attachment); 
                // ScriptManager.RegisterStartupScript(Page, Page.GetType(), "zobrazdokument", string.Format("ksShowEditor('{0}');", id), true);
                RE_editor.Visible = true;
                ScriptManager.RegisterStartupScript(Page, Page.GetType(), "zobrazdokument", string.Format("ksShowEditor();"), true);

                //JO 23.08.2021 - doplněn volitelný text tlačítka
                if (!string.IsNullOrEmpty(bull.tlacitko))
                {
                    TRB_Zkontrolovano.Text = bull.tlacitko;
                }
                else
                {
                    TRB_Zkontrolovano.Text = ReadGlobalResource.ZmenaTextu("[%Dokument zkontrolován%]");
                }


                TRB_Zkontrolovano.Visible = true;

                if (Editacni)
                {
                    TRB_Zkontrolovano.Visible = false;
                }
            }
            else
            {
                //PdfViewer1.Visible = false;
                //Btn_kontrola.Visible = false;
                RE_editor.Visible = true;
                RE_editor.Content = html;

                RPV_Viewer.Visible = false;
                TRB_Zkontrolovano.Visible = false;
                //ScriptManager.RegisterStartupScript(Page, Page.GetType(), "hideButton", string.Format("hideButton();"), true);
            }
            //KSProgram.Bulletin.Bulletin bull_stats = KSProgram.Bulletin.Bulletin.GetBulletins(id, DateTime.Now, Globals.OsCisloAktUserNeboZastupovany, null, true).FirstOrDefault();
            if (bull.Stats.Count > 0)
            {
                if (bull.Stats[0].precteno.IsNullOrEmpty())
                {
                    TRB_Zkontrolovano.Visible = true;
                }
            }
        }

        public void SaveBulletin()
        {
            if (IdBull.HasValue)
            {
                KSProgram.Bulletin.Bulletin bull = KSProgram.Bulletin.Bulletin.GetBulletins(IdBull.Value, null, null, null).FirstOrDefault();
                bull.Html = RE_editor.Content;
                bull.Save();
            }
        }

        public void ShowWindow()
        {
            Opened = true;
            RW_editor.VisibleOnPageLoad = true;
        }

        public void CloseWindow()
        {
            Opened = false;
            RW_editor.VisibleOnPageLoad = false;
        }

        protected void TRB_editor_save_Click(object sender, EventArgs e)
        {
            SaveBulletin();
        }

        private void creatreDirStructure()
        {
            string rootDir = AppDomain.CurrentDomain.BaseDirectory;
            string logsDir = rootDir + @"Images\Editor\";
            //string logsDir = rootDir + @"Temp\";
            createPath(logsDir);
            //rootDir + @"Temp"
            //~/Images/Editor

            string path = "~/Images/Editor"; //"~/Temp" "~/Images/Editor"

            RE_editor.ImageManager.ViewPaths = new string[] { path };
            RE_editor.ImageManager.UploadPaths = new string[] { path };
            RE_editor.ImageManager.DeletePaths = new string[] { path };
            RE_editor.ImageManager.EnableAsyncUpload = true;
            RE_editor.ImageManager.MaxUploadFileSize = 10485760;

            RE_editor.DocumentManager.ViewPaths = new string[] { path };
            RE_editor.DocumentManager.UploadPaths = new string[] { path };
            RE_editor.DocumentManager.DeletePaths = new string[] { path };
            RE_editor.DocumentManager.EnableAsyncUpload = true;
            RE_editor.DocumentManager.MaxUploadFileSize = 10485760;

            RE_editor.MediaManager.ViewPaths = new string[] { path };
            RE_editor.MediaManager.UploadPaths = new string[] { path };
            RE_editor.MediaManager.DeletePaths = new string[] { path };
            RE_editor.MediaManager.EnableAsyncUpload = true;
            RE_editor.MediaManager.MaxUploadFileSize = 10485760;

            RE_editor.FlashManager.ViewPaths = new string[] { path };
            RE_editor.FlashManager.UploadPaths = new string[] { path };
            RE_editor.FlashManager.DeletePaths = new string[] { path };
            RE_editor.FlashManager.EnableAsyncUpload = true;
            RE_editor.FlashManager.MaxUploadFileSize = 10485760;

            RE_editor.TemplateManager.ViewPaths = new string[] { path };
            RE_editor.TemplateManager.UploadPaths = new string[] { path };
            RE_editor.TemplateManager.DeletePaths = new string[] { path };
            RE_editor.TemplateManager.EnableAsyncUpload = true;
            RE_editor.TemplateManager.MaxUploadFileSize = 10485760;

            RE_editor.SilverlightManager.ViewPaths = new string[] { path };
            RE_editor.SilverlightManager.UploadPaths = new string[] { path };
            RE_editor.SilverlightManager.DeletePaths = new string[] { path };
            RE_editor.SilverlightManager.EnableAsyncUpload = true;
            RE_editor.SilverlightManager.MaxUploadFileSize = 10485760;
        }
        private void createPath(string path)
        {
            if (!Directory.Exists(path))
                Directory.CreateDirectory(path);
        }

        protected void RAU_file_FileUploaded(object sender, Telerik.Web.UI.FileUploadedEventArgs e)
        {
            try
            {
                string fileExt = e.File.GetExtension();
                switch (fileExt)
                {
                    case ".doc":
                    case ".docx":
                        RE_editor.LoadDocxContent(e.File.InputStream);
                        break;
                    case ".rtf":
                        RE_editor.LoadRtfContent(e.File.InputStream);
                        break;
                    case ".txt":
                    case ".html":
                    case ".htm":
                        using (StreamReader sr = new StreamReader(e.File.InputStream))
                        {
                            RE_editor.Content = sr.ReadToEnd();
                        }
                        break;
                    case ".md":
                        using (StreamReader sr = new StreamReader(e.File.InputStream))
                        {
                            RE_editor.Content = sr.ReadToEnd();
                            ScriptManager.RegisterStartupScript(Page, Page.GetType(), "importMarkdownScript", "BullEditor.setMarkdownContent();", true);
                        }
                        break;
                    default:
                        RadNotification1.Show("The selected file is invalid. Please upload an MS Word document with an extension .doc, .docx or .rtf, or a .txt/.html file with HTML content!");
                        break;
                }
            }
            catch (Exception ex)
            {
                RadNotification1.Show("There was an error during the import operation. Try simplifying the content.");
            }
        }

        protected void TRB_Zkontrolovano_Click(object sender, EventArgs e)
        {
            int id_bull = Convert.ToInt32(HF_id.Value);
            KSProgram.Bulletin.BulletinExtensions.SaveReadStats(id_bull);
            CloseWindow();
            Response.Redirect("~/Bulletin/Bulletin.aspx");
        }
    }
}