﻿using System;
using System.Data;
using System.Web.UI;
using System.Web.UI.WebControls;
using KSProgram.Pozadavky;
using KSProgram.WebControls;
using KSProgram;
using KSProgram.Data;
using KSProgram.Funkce;
using Telerik.Web.UI;
using KSProgram.Extensions;
using KSProgram.Obecne;
using System.Collections.Generic;
using KSProgram.Exceptions;
using KSProgram.Security;
using System.Text.RegularExpressions;

namespace KSPortal.Controls
{
    public partial class ZadostOzroczuct : UserControlPozadavek
    {
        /// <summary>
        /// cesta k usercontrolu (k variantě formuláře), který se má načíst při postbacku
        /// </summary>
        protected string UserControlPostBack
        {
            get
            {
                return (ViewState["UCP_subctr"] != null ? ViewState["UCP_subctr"].ToString() : "");
            }
            set
            {
                ViewState["UCP_subctr"] = value;
            }
        }

        protected FormViewMode CurrentMode
        {
            get
            {
                return (ViewState["curmode"] != null ? (FormViewMode)ViewState["curmode"] : FormViewMode.ReadOnly);
            }
            set
            {
                ViewState["curmode"] = value;
            }
        }

        public override void ChangeMode(FormViewMode mode)
        {
            CurrentMode = mode;
        }

        private void ChangeSubCtrlType(string subtyp, int? rok)
        {
            if (string.IsNullOrEmpty(subtyp))
            {
                if ((Zakaznik.IsCurrent("TIETO") || Globals.RAActive("DANOVKA_2")))
                {
                    if (PozId == 0)
                        subtyp = "KS";
                    else if (WF.FazePosl == 1 && WF.isVydavatel() && WF.Get_UN("", "Form1", "Mode")=="edit") //pokud je zadavani az ve 2. kroku (a v 1. kroku je jen "chci RZ"/"Nechci RZ")
                        subtyp = "KS";
                    else
                        subtyp = "MF";
                }
                else
                    subtyp = "MF";
            }

            if (!string.IsNullOrEmpty(subtyp) && rok != null)
            {
                string version;
                if (subtyp == "KS")
                {
                    version = "2021";
                }
                else //if (subtyp == "MF")
                {
                    if (rok >= 2022) //PR 22.12.2022 
                        version = "2022";
                    else if (rok == 2021)
                        version = "2021";
                    else
                        version = "2018";
                }

                UserControlPostBack = @"~/Controls/ZRZ_" + subtyp + "_" + version + ".ascx";
            }
        }

        private UserControlPozadavek LoadSubControl()
        {            
            if (!string.IsNullOrEmpty(UserControlPostBack))
            {
                UserControlPozadavek subctrl = null;
                subctrl = (UserControlPozadavek)LoadControl(UserControlPostBack);
                subctrl.ID = "subctrl";
                subctrl.Page = this.Page;
                // subctrl.InitLoad(this, new EventArgs());//inicializace controlky
                ph_subctrl.Controls.Clear();
                ph_subctrl.Controls.Add(subctrl);
                if (subctrl != null)
                {
                    subctrl.ItemInserted += new EventHandler(ctrl_ItemInserted);
                    //subctrl.ItemExit += new EventHandler(ctrl_ItemExit);
                    //subctrl.ItemRetrieved += new EventHandler(ctrl_ItemRetrieved);
                    subctrl.ButtonClicked += new EventHandler(ctrl_ButtonClicked);
                }

                return subctrl;
            }
            return null;

        }

        //insert přes ODS neprobíhá
        //protected void ObjectDataSource1_Inserted(object sender, ObjectDataSourceStatusEventArgs e)
        //{
        //    PozId = Convert.ToInt32(e.OutputParameters["id_poz"]);
        //}

        void ctrl_ItemInserted(object sender, EventArgs e)
        {
            PozId = SubCtrl.PozId;
        }

        void ctrl_ButtonClicked(object sender, EventArgs e)
        {
            if (((Control)sender).ID == "RB_k_podpisu" /*&& Page.IsValid kvuli nevyplnenemu stavu me to nepusti*/)
            {
                //Načtu data z ZRZ_KS_2021
                System.Collections.Generic.Dictionary<string, string> values = new System.Collections.Generic.Dictionary<string, string>();
                foreach (DataColumn c in Pozadavekzroczuct.GetDataTable().Columns)
                {
                    if (null == WebControlHelper.FindControl(SubCtrl, c.ColumnName))
                        values[c.ColumnName] = "IGNORE";
                    else
                        values[c.ColumnName] = WebControlHelper.SubCtrlGetValueString(SubCtrl, c.ColumnName, true, c.ColumnName);
                }

                //Přehodím SubCtrl z typu ZRZ_KS_2021 na ZRZ_MF_2018
                ChangeSubCtrlType(/*subtyp*/"MF", Int32.Parse(values["H_obd_rok"]));
                //LoadSubControl();
                //SubCtrl.Nacist(); //aby se nacetli udaje, ktere nejsou v ZRZ_KS_2021 a aby se disablovali polozky, ktere uzivatel nemuze editovat
                //PR 17.12.2021 nepřeneslo se ID do Subctr
                UserControlPozadavek subctr = LoadSubControl();
                if (subctr != null)
                {
                    subctr.PozId = PozId;
                    subctr.WF = WF;

                    subctr.Nacist();
                }

                //Naplním hodnoty do něj
                foreach (DataColumn c in Pozadavekzroczuct.GetDataTable().Columns)
                {
                    if (values[c.ColumnName] != "IGNORE")
                        WebControlHelper.SubCtrlSetValue(SubCtrl, c.ColumnName, values[c.ColumnName]);
                }
            }
        }

        public override void Nacist()
        {
            int rok;
            if (PozId > 0)
                rok = Pozadavekzroczuct.Rok(PozId);
            else 
                rok = DateTime.Today.Year - 1;

            ChangeSubCtrlType(/*subtyp*/null, rok);            
            
            UserControlPozadavek subctr = LoadSubControl();
            if (subctr != null)
            {
                subctr.PozId = PozId;
                subctr.WF = WF;

                subctr.Nacist();
            }
        }

        protected UserControlPozadavek SubCtrl
        {
            get
            {
                if (ph_subctrl.Controls == null && ph_subctrl.Controls.Count == 0)
                    return null;
                UserControlPozadavek c = (UserControlPozadavek)ph_subctrl.Controls[0];
                return c;
            }
        }

        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack)
            {

            }
            LoadSubControl();
        }

        /// <summary>
        /// Sem směřují všechny commandy z pozadavky.aspx
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public override void CommandRouter(object sender, CommandEventArgs e, ref bool is_valid)
        {
            if (e.CommandName.ToString() == "Save")
            {
                string form_visible = WF.Get_UN("", "Form1", "Visible");//PR 17.12.2021 

                if (SubCtrl is ZRZ_KS_2021 && (form_visible == "true" || form_visible == ""))
                {
                    val_podepsano.ErrorMessage = "Nelze uložit, nejprve přejděte k podepsání přes tlačítko PODEPSAT!";
                    val_podepsano.IsValid = false;
                    is_valid = false;
                }
                else //if (SubCtrl is ZRZ_MF_2018 || SubCtrl is ZRZ_MF_2021)
                {
                    SubCtrl.CommandRouter(sender, e, ref is_valid);
                    this.PozId = SubCtrl.PozId;//PR 17.12.2021 
                }
            }
        }

    }
}