﻿using KSProgram;
using KSProgram.Data;
using KSProgram.Extensions;
using KSProgram.Obecne;
using KSProgram.WebControls;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Telerik.Web.UI;
using System.Web.Script.Serialization;
using KSProgram.InfoBlob;

namespace KSPortal.DashBoard
{
    public partial class DashBoard : System.Web.UI.UserControl
    {
        public List<DashBoardPanel> ListOfWidgets
        {
            get
            {
                return new List<DashBoardPanel>()
                {
                    new DashBoardPanel_Bulletin(),
                    new DashBoardPanel_Odkazy(),
                    new DashBoardPanel_Pozadavky(),
                    new DashBoardPanel_SchvaleneNepritomnosti(),
                    //new DashBoardPanel_Warnigs(),
                    new DashBoardPanel_VolnePozice(),
                    new DashBoardPanel_Ankety(),
                    new DashBoardPanel_NovyPozadavek()
                };
            }
        }

        public List<RadDockZone> DockZones
        {
            get
            {
                return new List<RadDockZone>()
                {
                    RDZ1, RDZ2, RDZ3, RDZ4, RDZ5, RDZ6
                };
            }
        }

        //JH 9.6.2022 - změna kůli pořadí DashBoardů
        protected void Page_Init(object sender, EventArgs e)
        {
            List<DashBoardPanel> dostupne = ListOfWidgets.FindAll(x => x.JeDostupny);
            string path;

            //DashBoardLayout layaut;
            if (dostupne.Exists(x => x is DashBoardPanel_Bulletin))
            {
                path = "~/DashBoard/DashBoardLayout1.ascx";
            }
            else
            {
                path = "~/DashBoard/DashBoardLayout2.ascx";
            }

            //layaut = (DashBoardLayout)LoadControl(path);

            List<int> volne = new List<int>();
            List<string> nezarazene = new List<string>();
            foreach (DashBoardPanel panel in dostupne)
            {
                nezarazene.Add(panel.Name);
            }

            string sel = "select id from info_blob where pouziti = 108 and os_cislo =" + Globals.OsCisloAktUser;
            string id = SqlPrikaz.ExecuteScalarString(sel, "-1");
            string json = null;

            if (id != "-1") json = InfoBlob.ReadString(int.Parse(id));

            if (!String.IsNullOrEmpty(json))
            {
                JavaScriptSerializer serializer = new JavaScriptSerializer();
                var jsonObject = serializer.Deserialize<dynamic>(json);

                for (int i = 0; i < 6; i++)
                {
                    if (jsonObject != null)
                    {
                        var docks = jsonObject[i][1];
                        RadDockZone dz = DockZones[i];
                        if (docks.Length > 0)
                        {
                            foreach(System.Collections.Generic.Dictionary<string, object> dock in docks)
                            {
                                string name = DockName(nezarazene, dock["Name"].ToString());
                                LoadWidget(dz, name);
                                nezarazene.Remove(name);
                            }                            
                        }
                        else
                        {
                            volne.Add(i);
                        }
                    }
                    else
                    {
                        volne.Add(i);
                    }
                }
            }
            else
            {
                for (int i = 0; i < 6; i++)
                {
                    volne.Add(i);
                }
            }

            foreach (string name in nezarazene)
            {
                if (volne.Count > 0)
                {
                    RadDockZone dzp = DockZones[volne[0]];
                    LoadWidget(dzp, name);
                    volne.Remove(volne[0]);
                }
                else
                {
                    RadDockZone dzp = DockZones[5];
                    LoadWidget(dzp, name);
                }
            }

            TRegisteredScripts.AddCssStyleToPage(this.Page, "~/DashBoard/css/DashBoard.css");
        }

        private RadDock CreateRadDock()
        {
            RadDock dock = new RadDock();
            dock.DockMode = DockMode.Docked;
            dock.UniqueName = Guid.NewGuid().ToString().Replace("-", "a");
            dock.ID = string.Format("RadDock{0}", dock.UniqueName);
            dock.Text = string.Format("Added at {0}", DateTime.Now);
            dock.Page = this.Page;

            //dock.Commands.Add(new DockCloseCommand());
            //dock.Commands.Add(new DockExpandCollapseCommand());
            //dock.Commands.Clear();

            DockCommand cmd = new DockCommand();
            cmd.CssClass = "hidden";
            dock.Commands.Add(cmd);

            //JH 11.10.2022
            dock.EnableAnimation = true;
            dock.AutoPostBack = false;
            dock.EnableDrag = true;
            dock.Style.Add("margin-bottom", "25px");
            dock.OnClientDockPositionChanged += "ulozLayout";

            return dock;
        }

        private void LoadWidget(RadDockZone dz, string nazev)
        {
            if (!string.IsNullOrEmpty(nazev))
            {
                RadDock dock = CreateRadDock();
                DashBoardPanel panel = (DashBoardPanel)LoadControl(string.Format("~/DashBoard/DashBoardPanel_{0}.ascx", nazev));
                if (panel.JeDostupny)
                {
                    if (panel.MinWidth != null)
                    {
                        LayoutColumn layoutColumn = dz.FindParentControl<LayoutColumn>();
                        layoutColumn.Style.Add("min-width", panel.MinWidth.ToString());
                    }

                    dock.ContentContainer.Controls.Add(panel);

                    dock.Title = panel.Nadpis;
                    dock.Tag = nazev;
                    dock.ID = nazev;

                    //dock.OnClientDockPositionChanged += new DockPositionChangedEventHandler(SavePosition);
                    dock.DockPositionChanged += new DockPositionChangedEventHandler(SavePosition);

                    RDL.Controls.Add(dock);
                    dock.Dock(dz);
                    //dock.DockZoneID = dz.ClientID;

                    if (string.IsNullOrEmpty(dock.Tag) || dock.Closed)
                    {
                        return;
                    }
                }
            }
        }

        private string DockName(List<string> nezarazene, string name)
        {
            foreach(string str in nezarazene)
            {
                if (name.Contains(str)) return str;
            }
            return null;
        }

        protected void SavePosition(object sender, DockPositionChangedEventArgs e)
        {
            string logString = string.Format("{0} is in {1} with index {2}<br/>" +
                "", ((Control)sender).ID, e.DockZoneID, e.Index);

            //RadDockZone dz = (RadDockZone)FindControl(e.DockZoneID.Substring(e.DockZoneID.Length - 4));
        }
    }
}