﻿using KSProgram;
using KSProgram.Bulletin;
using KSProgram.Data;
using KSProgram.Dokument;
using KSProgram.Extensions;
using KSProgram.Obecne;
using KSProgram.Pozadavky;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Web;
using System.Web.Services;
using System.Web.UI;
using System.Web.UI.WebControls;
using Telerik.Web.UI;

namespace KSPortal.DashBoard
{
    public partial class WebMethods : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {

        }

        [WebMethod]
        public static DataResult GetLokace()
        {
            List<DynamicDataObject> data = GetSouradniceZamestnancu();

            var definice = new
            {
                os_cislo = 0m,
                jmeno = "",
                adresa = "",
                location = new double[] { 0, 0 },
                kraj = "",
                mesto = "",
            };

            return new DataResult
            {
                Data = from d in data.ToEnumarable(definice)
                       select (object)d,
                Count = data.Count
            };
        }

        [WebMethod]
        public static DataResult GetMesta()
        {
            List<DynamicDataObject> data = GetSouradniceZamestnancu();

            return new DataResult
            {
                Data = (from d in data
                        group d by d.Get("mesto")?.ToString()
                        into grouped
                        select ((object)new
                        {
                            item = grouped.Key,
                            value = grouped.Count(),
                            location = new double[2] { grouped.Average(x => ((double[])x.Get("location"))[0]), grouped.Average(x => ((double[])x.Get("location"))[1]) },
                        })).ToList(),
                Count = data.Count
            };
        }

        public static List<DynamicDataObject> GetSouradniceZamestnancu()
        {
            string cachekey = "mapaZamestnancu";
            var data = (new[] { new { os_cislo = (decimal?)0, jmeno = "", adresa = "", location = new double[] { 0, 0 }, kraj = "", mesto = "" } }).AsEnumerable();
            string base64 = CacheInfo.GetItemFromCache(cachekey)?.ToString();
            if (base64 == null)
            {
                string sql = "SELECT os_cislo, jmeno, prijmeni, pr_ulice, pr_cislo, psc_prech, misto_prech, prech_stat_adr, " +
                                       "tr_ulice, tr_cislo, psc_trv, misto_trv, stat_adr " +
                                       "FROM pracovnici";
                ParametryPrikazu pars = new ParametryPrikazu() { };
                DataTable zamci = SqlPrikaz.ExecuteCommand(sql, true, true, ref pars);


                double[] loc = new double[] { 0, 0 };
                string m, k;
                data = from r in zamci.AsEnumerable()
                       select (new
                       {
                           os_cislo = GetNullOrData.GetDecimal(r["os_cislo"]),
                           jmeno = GetNullOrData.GetString(r["jmeno"], "", true) + " " + GetNullOrData.GetString(r["prijmeni"], "", true),
                           adresa = getLokace(string.IsNullOrEmpty(GetNullOrData.GetString(r["misto_prech"])) ?
                                    (GetNullOrData.GetString(r["tr_ulice"], "", true) + " " + GetNullOrData.GetString(r["tr_cislo"], "", true) + ", " + GetNullOrData.GetString(r["psc_trv"], "", true) + " " + GetNullOrData.GetString(r["misto_trv"], "", true) + (string.IsNullOrEmpty(GetNullOrData.GetString(r["stat_adr"])) ? "" : (", " + GetNullOrData.GetString(r["stat_adr"], "", true)))) :
                                    (GetNullOrData.GetString(r["pr_ulice"], "", true) + " " + GetNullOrData.GetString(r["pr_cislo"], "", true) + ", " + GetNullOrData.GetString(r["psc_prech"], "", true) + " " + GetNullOrData.GetString(r["misto_prech"], "", true) + (string.IsNullOrEmpty(GetNullOrData.GetString(r["prech_stat_adr"])) ? "" : (", " + GetNullOrData.GetString(r["prech_stat_adr"], "", true)))), out loc, out k, out m),
                           location = loc,
                           kraj = k,
                           mesto = m
                       });
                CacheInfo.AddItemToCache(cachekey, Newtonsoft.Json.JsonConvert.SerializeObject(data).ZipToBase64());
            }
            else
            {
                data = Newtonsoft.Json.JsonConvert.DeserializeAnonymousType(base64.UnZipFromBase64(), data);
            }

            List<DynamicDataObject> ret = new List<DynamicDataObject>() { };
            foreach (var d in data)
            {
                DynamicDataObject o = new DynamicDataObject();
                o.Set("os_cislo", d.os_cislo);
                o.Set("jmeno", d.jmeno);
                o.Set("adresa", d.adresa);
                o.Set("location", d.location);
                o.Set("kraj", d.kraj);
                o.Set("mesto", d.mesto);
                ret.Add(o);
            }
            return ret;
        }

        private static string getLokace(string adresa, out double[] location, out string kraj, out string mesto)
        {
            string ret = "";
            kraj = ""; mesto = "";
            location = new double[] { 0, 0 };
            string apikey = "AIzaSyB46Ndy-gPSj7KWhiW0vbJm9TZkX8Kq9Ec";
            var definition = new { results = new[] { new { address_components = new[] { new { long_name = "", short_name = "", types = new[] { "" } } }, formatted_address = "", geometry = new { location = new { lat = (double)0, lng = (double)0 } } } }.ToList() };

            HttpWebRequest request = null;
            HttpWebResponse response = null;
            StreamReader reader = null;

            try
            {
                string adrkey = adresa;
                string url = string.Format("https://maps.googleapis.com/maps/api/geocode/json?address={0}&key={1}", adrkey, apikey);
                request = (HttpWebRequest)WebRequest.Create(url);
                response = (HttpWebResponse)request.GetResponse();
                reader = new StreamReader(response.GetResponseStream());
                string json = reader.ReadToEnd();
                var o = Newtonsoft.Json.JsonConvert.DeserializeAnonymousType(json, definition);

                if(o != null && o.results != null && o.results.Count() > 0)
                {
                    mesto = o.results.FirstOrDefault()?.address_components.FirstOrDefault(x => x.types.Contains("administrative_area_level_2"))?.long_name;
                    kraj = o.results.FirstOrDefault()?.address_components.FirstOrDefault(x => x.types.Contains("administrative_area_level_1"))?.long_name;

                    location = new double[2] { o.results.FirstOrDefault().geometry.location.lat, o.results.FirstOrDefault().geometry.location.lng };
                    return o.results.FirstOrDefault()?.formatted_address;
                }
            }
            catch (Exception e)
            {

            }
            return ret;
        }

        [WebMethod]
        public static DataResult GetBulletins()
        {
            if (KSProgram.Prava.ZamykaniAgend.GeneralZamekIsLocked())
            {
                return null;
            }

            var list = KSProgram.Bulletin.Bulletin.GetBulletins(null, DateTime.Now, Globals.OsCisloAktUserNeboZastupovany, true, true, true);

            return new DataResult
            {
                Data = list,
                Count = list.Count
            };
        }


        [WebMethod]
        public static DataResult GetWarnigs()
        {
            var list = (new KSProgram.Obecne.WarningMessages()).GetAllMessages().Select(x => new { message = x }).ToList();

            return new DataResult
            {
                Data = list,
                Count = list.Count
            };
        }

        [WebMethod]
        public static DataResult GetOdkazy()
        {
            var list =
                (new KSProgram.Odkazy()).Seznam()
                .ToList(new
                {
                    nazev = "",
                    url = "",
                    cil = "",
                    ikona = ""
                })
                .Select(x => new
                {
                    nazev = x.nazev.Trim(),
                    url =  x.url.StartsWith("~") ? (Pozadavek.UrlBaseInterni().Trim() + x.url.Remove(0, 1)) : x.url,
                    cil = x.cil,
                    ikona = x.ikona
                })
                .ToList();



            return new DataResult
            {
                Data = list,
                Count = list.Count
            };
        }

        [WebMethod]
        public static DataResult GetVolnePozice()
        {
            var list = (new KSProgram.VyberovaRizeni.VyberovaRizeni()).Publikovane(true, false)
                .ToList(new { id_vybriz = 0, nazev = "" })
                .Select(x => new
                {
                    url = KSProgram.VyberovaRizeni.VyberovaRizeni.VolnaPoziceBuildURL(x.id_vybriz),
                    nazev = x.nazev
                })
                .ToList();

            return new DataResult
            {
                Data = list,
                Count = list.Count
            };
        }

        [WebMethod]
        public static DataResult GetNepritomnosti()
        {
            var list = (new KSProgram.Sestavy.SchvaleneNeprit()).Retrieve()
                .ToList(new { os_cislo = 0m, cislo_pom = (short)0, jmeno = "", d_1 = "", d_2 = "", d_3 = "", d_4 = "", d_5 = "", d_6 = "", d_7 = "", d_8 = "", d_9 = "", d_10 = "" });

            return new DataResult
            {
                Data = list,
                Count = list.Count
            };
        }

        [WebMethod(EnableSession = true)]
        public static string GetBulletinHtml(int id)
        {
            KSProgram.Bulletin.Bulletin bull = KSProgram.Bulletin.Bulletin.GetBulletins(id, null, null, null, false)?.FirstOrDefault();
            IEnumerable<KSProgram.Bulletin.Bulletin> bulls = new List<KSProgram.Bulletin.Bulletin> { bull };

            byte[] data = bull?.LoadHtmlByte();
            string html = bull?.LoadHtml();
            byte[] pdf;
           
            if (PDF.IsValidPdf(data))
            {
                bulls.SaveSatas(true, false);
                return "PDF" + Convert.ToBase64String(data);
            }
            else if (html.TryParseBase64(out pdf) && PDF.IsValidPdf(pdf))
            {
                bulls.SaveSatas(true, false);
                return "PDF" + html;
            }
            else if (bull.id_telo.HasValue && !string.IsNullOrEmpty(html))
            {
                bulls.SaveSatas(true, false);
                return html;
            }
            else
            {
                return "neni telo";
            }


            //try
            //{
            //    PdfReader reader = new PdfReader(data);
            //}
            //catch
            //{
            //    string html = KSProgram.Bulletin.Bulletin.GetBulletins(id, null, null, null, false)?.FirstOrDefault()?.LoadHtml();
            //    try
            //    {
            //        byte[] pdf = Convert.FromBase64String(html);
            //        PdfReader reader = new PdfReader(pdf);
            //        bulls.SaveSatas(true, false);
            //        return "PDF" + html;
            //    }
            //    catch
            //    {
            //        bulls.SaveSatas(true, false);
            //        return html;
            //    }

            //}
            
        }

        [WebMethod]
        public static string GetTextTlacitko(int id)
        {
            KSProgram.Bulletin.Bulletin bull = KSProgram.Bulletin.Bulletin.GetBulletins(id, null, null, null, false)?.FirstOrDefault();

            string text_tlacitka = bull.tlacitko;

            if (string.IsNullOrEmpty(text_tlacitka))
            {
                return ReadGlobalResource.ZmenaTextu("[%Dokument zkontrolován%]");
            }

            return text_tlacitka;
        }

        [WebMethod(EnableSession = true)]
        public static bool CallPdfUpdate(int id)
        {
            return KSProgram.Bulletin.BulletinExtensions.SaveReadStats(id);

        }

        [WebMethod]
        public static bool ZkontrolujDatumPotvrzeni(int id)
        {
            KSProgram.Bulletin.Bulletin bull = KSProgram.Bulletin.Bulletin.GetBulletins(id, null, null, null, true,true)?.FirstOrDefault();
            
            if(!bull.Stats[0].precteno.IsNullOrEmpty())
            {
                return true;
            }

            return false;
        }

        [WebMethod]
        public static DataResult GetPozadavky()
        {
            PozadavekSeznam sez = new PozadavekSeznam();
            List<object> ret = (from r in (from p in sez.Retrieve(0, "vse", "vse", "mekvyrizeni", DateTime.MinValue, DateTime.MaxValue, DateTime.Today.AddMonths(-1).PrvniDen(), DateTime.MaxValue, null, "vse", "n,r").AsEnumerable()
                                           select new PozadavekSeznam.Pozad(p))
                                orderby r.adr_ja ascending, r.datum ascending
                                select ((object)new pozadavek()
                                {
                                    id_poz = r.id_poz,
                                    prijmeni = r.prijmeni,
                                    jmeno = r.jmeno,
                                    typ = ReadGlobalResource.ZmenaTextu(r.typ),
                                    text = ReadGlobalResource.ZmenaTextu(r.text),
                                    stav = r.stav,
                                    url = Pozadavek.BuildURLPozadavkuDetailInterni((int)r.id_poz),
                                    ikona = r.ikona()
                                })).ToList();

            return new DataResult
            {
                Data = ret,
                Count = ret.Count
            };
        }

        //JH 9.6.2022
        [WebMethod]
        public static string UlozLayout(string obj, string data)
        {
            try
            {
                string sqlDel = "delete from info_blob where pouziti = 108 and os_cislo=" + Globals.OsCisloAktUser;
                SqlPrikaz.ExecuteCommand(sqlDel);

                //KSProgram.Obecne.ObjSetting.SaveValue(Globals.OsCisloAktUser, obj, "json", data);
                int? id = null;
                SqlPrikaz.SaveTextToInfoBlob(ref id, Globals.OsCisloAktUser, null, 108, DateTime.Now, null, data, true, null, null, "json", false);
            }
            catch(Exception e)
            {
                return e.Message;
            }           
            return "ok";
        }

        //JH 25.1.2023
        [WebMethod]
        public static DataResult GetNovePoz()
        {
            var list =
                PozadavkyTypy.GetTypy(false, false, false)
                .ToList(new
                {
                    typ_text = "",
                    typ = "",
                    ikona = ""
                })
                .Select(x => new
                {
                    nazev = ReadGlobalResource.ZmenaTextu(x.typ_text.Trim()),
                    url = Pozadavek.UrlBaseInterni().Trim() + Pozadavek.BuildURLPozadavkuTypDetail(x.typ).Remove(0, 1),
                    ikona = PozadavkyTypy.GetIcon(x.typ)
                })
                .ToList();

            return new DataResult
            {
                Data = list,
                Count = list.Count
            };
        }

        public class DataResult
        {
            public IEnumerable<object> Data { get; set; }
            public int Count { get; set; }
        }

        public class PozadavkyDataResult
        {
            public List<object> Data { get; set; }
            public int Count { get; set; }
        }
        public class pozadavek
        {
            public int? id_poz { get; set; }
            public string jmeno { get; set; }
            public string prijmeni { get; set; }
            public string typ { get; set; }
            public string text { get; set; }
            public string stav { get; set; }
            public string url { get; set; }
            public string ikona { get; set; }
        }
    }
}