﻿using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using KSProgram;
using KSProgram.Prava;
using KSProgram.Data;
using System.Collections.Generic;
using System.Diagnostics;
using KSProgram.Diagnostics;
using KSProgram.WebControls;
using KSProgram.Pozadavky;
using KSProgram.Exceptions;
using KSProgram.Extensions;
using KSProgram.Funkce;
using Telerik.Web.UI;
using KSProgram.Obecne;
using KSProgram.Security;
using System.Web.Configuration;
using System.Collections.Specialized;
using System.Data.Odbc;
using System.Data.OracleClient;
using System.Data.SqlClient;
using System.Net;
using KSProgram.Uzivatele;
using KSProgram.Email;
using System.Web.Services;
using Newtonsoft.Json.Linq;

namespace KSPortal
{
    public partial class Login : TPrePage
    {
        public Login()
        {
            IsLoginPage = true;
        }
        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);
            F_Telerik.AddSkinReferenceLinks(Page);
        }

        protected override void Construct()
        {
            base.Construct();
            //PR 08.06.2020 Ochrana proti Session Fixation (útočník se přihlásí, útočník podvrhne oběti SessionId, oběť se přihlásí, nyní sdílejí stejnou session, ve ktere je login, práva..)
            //Zároveň to řeší i problém, kdy se v prohlížeči usadí více cookies "ASP.NET_SessionId" a ASP pořád generuje nové session a znefunkční to přihlášení.
            //HttpContext.Current.Session.Abandon() nemuzu pouzit, protoze zde neni dostupna a prisel bych o udaje v session Global.User a dalsi
            //Zneplatneni cookiny/vynulovani nemuzu udelat v jine metode, kde je jiz objekt Session vytvoren dle session id v cookine ASP.NET_SessionId,
            //tim bych zneplatnil session, do ktere bude v zapeti ulozeno prihlasovaci jmeno (Global.User).
            if (HttpContext.Current.Request.HttpMethod == "POST"/*IsPostBack*/)
            {
                if (HttpContext.Current.Request.Form["__EVENTTARGET"] != null)
                    if (HttpContext.Current.Request.Form["__EVENTTARGET"].EndsWith("LoginButton"))
                    {
                        //protože se mi někdy záhadně v prohlížeči usadili 2 nebo 3 cookiny se stejným názvem, raději cyklus (vice cookin znefunkční přihlášení)
                        if (HttpContext.Current.Request.Cookies != null)
                            for (int i = 0; i < HttpContext.Current.Request.Cookies.Count; i++)
                            {
                                if (HttpContext.Current.Request.Cookies[i].Name == "ASP.NET_SessionId")
                                {
                                    HttpContext.Current.Request.Cookies[i].Value = "";//zneplatnim ji, takze dostanu novou
                                    if (HttpContext.Current.Request.Cookies[i].Expires > DateTime.Now || HttpContext.Current.Request.Cookies[i].Expires == DateTime.MinValue)
                                        HttpContext.Current.Request.Cookies[i].Expires = DateTime.Now.AddMonths(-20);
                                }
                            }
                    }
            }
        }

        protected override void Page_Load(object sender, EventArgs e)
        {
            base.Page_Load(sender, e);

            //PR 24.02.2020 pokud již je autentizován, tak ho přesměřuji na Default.aspx, aby se nemohl přihlásit bez předchozího odhlášení
            if (Request.IsAuthenticated && !string.IsNullOrEmpty(Globals.User) && Globals.OsCisloAktUser != 0)
            {
                if (!Request.Url.Query.StartsWith("?ReturnUrl=%2fKSPortal%2f__browserLink%2frequestData"))//PR 24.02.2020 Visual Studio si něco čte z prohlížeče
                {
                    BrowserLabel.Text = "Již jste přihlášen!";
                    HttpContext.Current.Response.Redirect(FormsAuthentication.DefaultUrl, true);//nechci nový autentizační ticket, takže prostý redirect. ?ReturnUrl ignoruji
                    return;
                }
            }

            //PR 31.10.2022 jen pro vyvolání validací správného nastavení parametrů TypOvereni a ZdrojOsCisla
            if (SpravaUziv.TypOvereni == SpravaUziv.TypOvereniEnum.None || SpravaUziv.ZdrojOsCisla == SpravaUziv.ZdrojOsCislaType.KSPU)
            {
                //nic, kontrola proběhne uvnitř vlastností výše
            }

                //MD 30.10.2008 Zda povolit registraci uživatelů
                LB_Registrace.Visible = (AppConfiguration.GetAppSetting<string>("Registrace", null) == "user");

            //PR 29.05.2019 Zda povolit obnovu zapomenutého hesla
            string PasswordReset = AppConfiguration.GetAppSetting<string>("PasswordReset", "");
            LB_ResetPassword.Visible = (PasswordReset == "1" || PasswordReset == "2");

            //MD 27.8.2009 Zda povolit přihlášení externích uživatelů
            LB_ExternalUser.Visible = AppConfiguration.GetAppSetting<bool>("ExterniUziv", false);

            //MD 21.3.2014 při oveřování v AD není potřeba kontroloval přihlašovací údaje na SQL Injection
            //if (AppConfiguration.GetAppSetting<string>("TypOvereni", null) == "ad")
            if (SpravaUziv.TypOvereni == SpravaUziv.TypOvereniEnum.AD)
            {
                PasswordValidator_UserName.Enabled = false;
                PasswordValidator_Password.Enabled = false;
            }

            if (!IsPostBack)
            {
                //SSO
                //------------------------------------
                if (SpravaUziv.LoginType == SpravaUziv.LoginTypes.Windows)
                {
                    UserName.Text = SpravaUziv.WindowsLogin();
                    UserName.Enabled = false;
                    PasswordLabel.Visible = false;
                    Password.Visible = false;
                    PasswordRequired.Enabled = false;
                    LoginButton.Focus();
                    LB_Registrace.PostBackUrl = "~/Public/RegistraceSso.aspx";
                }
                else
                {
                    UserName.Focus(); //MD 19.3.2010
                }
                //------------------------------------

                //načtu seznam connectionstringu
                ConnectionStringSettingsCollection connectionStrings = DatabaseInfo.GetConnectionStrings(); //MD 17.10.2013
                for (int i = 0; i < connectionStrings.Count; i++)
                {
                    if (connectionStrings[i].Name.Contains("hidden_"))
                    { }
                    else
                    {
                        DropDownListItem item = new DropDownListItem(connectionStrings[i].Name, connectionStrings[i].Name);
                        ddl_databaze.Items.Add(item);
                    }
                }

                //MD 24.4.2014 Test prohlížeče
                //BrowserInfo browser = new BrowserInfo();
                //if (!browser.IsSupported())
                //{
                //    BrowserLabel.Text = browser.GetErrorMessage();

                //    //PR 30.06.2021 HID=56574
                //    if ("1" == AppConfiguration.GetAppSetting<string>("NotSupBrowser", "1"))
                //    {
                //        //PR 30.06.2021 
                //        BrowserLabel.Text += ReadGlobalResource.ZmenaTextu(". [%Použij aktuální verzi jednoho z následujících prohlížečů%]") + ": Google Chrome, Microsoft Edge, Mozilla Firefox, Opera";
                //        BrowserLabel.Font.Size = new FontUnit("18px");
                //        LoginPanel.Visible = false;
                //        LoginButton.Enabled = false;
                //    }
                //}

                //a vyberu posledně použitou
                if (Request.Cookies["UserSettings"] != null)
                {
                    string volbaDb;
                    if (Request.Cookies["UserSettings"]["volbaDb"] != null)
                    {
                        volbaDb = Request.Cookies["UserSettings"]["volbaDb"];
                        ddl_databaze.SelectedValue = volbaDb;
                    }
                }

                //PR 22.2.2017 předvyplnění databáze dle dbname v ?encq= (URL na workflow požadavek obsahující idpoz a dbname)
                KSProgram.QueryStringCrypto hh = new KSProgram.QueryStringCrypto();
                NameValueCollection queryString = hh.DecryptRequestQuery();
                if (queryString.Count > 0 && !String.IsNullOrEmpty(queryString["dbname"]))
                {
                    string db = queryString["dbname"];
                    if (null == ddl_databaze.Items.FindChildByValue<DropDownListItem>(db, false))
                        MessageBox.Alert(this, "Název databáze " + db + " předaný URL odkazem není v nabídce databází!");
                    else
                        ddl_databaze.SelectedValue = queryString["dbname"];
                }
            }

            //Pokud se využívá SSO a má se přeskočit LoginPage, spustím automaticky přihlášení
            bool skipLoginPage = AppConfiguration.GetAppSetting<bool>("SkipLoginPage", false); //MD 27.5.2013
            if (SpravaUziv.LoginType == SpravaUziv.LoginTypes.Windows && skipLoginPage)
            {
                if (Request.Url.Query.StartsWith("?ReturnUrl=%2fKSPortal%2f__browserLink%2frequestData"))//PR 24.02.2020 Visual Studio si něco čte z prohlížeče
                    return;
                LoginButton_Click(this, new EventArgs());
            }

            HF_NotSupBrowser.Value = AppConfiguration.GetAppSetting<string>("NotSupBrowser", "1");
        }
        protected void LoginButton_Click(object sender, EventArgs e)
        {
            KSTrace.WriteLine("LoginButton_Click start");
            //PR 11.12.2019 přepsal jsem auditování přihlášení
            bool prihlasen = false;

            Globals.ClearDb(); //PH 20.6.2007 musím odstranit odkaz na databázi

            string login = UserName.Text.Trim();
            string password = Password.Text.Trim();

            string dbName = string.Empty;
            if (ddl_databaze.SelectedItem == null)
            {
                FailureText.ErrorMessage = ReadGlobalResource.ZmenaTextu("[%Není vybrána databáze%]");
                FailureText.IsValid = false;
                return;
            }
            else
            {
                dbName = ddl_databaze.SelectedItem.Text;
                //zapamatuju si vybranou databázi
                HttpCookie myCookie = new HttpCookie("UserSettings");
                myCookie["volbaDb"] = dbName;
                myCookie.Expires = DateTime.Now.AddDays(30d);

                //Response.Cookies.Add(myCookie); MIK 9.1.2014
                F_Cookie.Add(myCookie);
            }

            //ověřuju zda má přístup
            //-----------------------------------------------------------
            try
            {
                if (SpravaUziv.LoginType == SpravaUziv.LoginTypes.Windows)
                {
                    login = SpravaUziv.WindowsLogin();
                    SpravaUziv.PrihlaseniOverenehoUzivatele(dbName);
                }
                else
                {
                    login = SpravaUziv.FormPrihlaseni(dbName, login, password); //PR 06.02.2019 
                }
                //PR 14.10.2014 inicializace proměnných v Globals pro přihlášeného uživatele (OsCisloAktUser,IsExternalUser,IsAdmin,Prava,Zastup,Zakaznik,ViceIco, ...)
                SpravaUziv.InicializaceUzivatele(login);

                prihlasen = true;
            }
            catch (LoginException loginExc)
            {
                string message = ReadGlobalResource.ZmenaTextu(loginExc.Message);
                LogonLogger.WriteLine(login, message);
                FailureText.ErrorMessage = message;
                FailureText.IsValid = false;

                if (loginExc.Code == -2) //expirace hesla
                {
                    LoginButton.Visible = false;
                    ChangePasswordButton.Visible = true;
                }
            }
            catch (System.Security.SecurityException securityExc)
            {
                string message = ReadGlobalResource.ZmenaTextu(securityExc.Message);
                LogonLogger.WriteLine(login, message);
                FailureText.ErrorMessage = message;
                FailureText.IsValid = false;
            }
            catch (KSProgram.AD.ADQueryException adExc)
            {
                KSProgram.ExceptionUtility.LogProcess(adExc); //MD 13.4.2016 Přidáno logování AD výjimky
                string message = ReadGlobalResource.ZmenaTextu(adExc.Message);
                LogonLogger.WriteLine(login, message);
                FailureText.ErrorMessage = message;
                FailureText.IsValid = false;
            }

            //MD 19.12.2014 Logování informací o cookies
            KSTrace.WriteLine("CookieMode", FormsAuthentication.CookieMode.ToString());
            KSTrace.WriteLine("CookiesSupported", FormsAuthentication.CookiesSupported.ToString());

            //PR 08.11.2022 pokud uživatel není přihlášen, tak se nemůže číst z databáze, nemusí být nastavena nebo to může být jiná DB v případě více DB v configu
            //              (a v případě sql autentizace ani neznám login a heslo k DB)
            if (Globals.IsDbConected && Globals.DBName == dbName)
            {
                //GDPR audit
                Uzivatel userinfo = new Uzivatel();
                userinfo.Initialize(dbName, login, Request);
                userinfo.SetGDPRAudit(prihlasen);
                Globals.UzivatelInfo = userinfo;
            }

            //PR 08.11.2022 pokud uživatel není přihlášen, tak se nemůže číst z databáze, nemusí být nastavena nebo to může být jiná DB v případě více DB v configu.
            //              A v případě TypOvereni=sql a neúspěšného přihlášení ani neznám login a heslo k DB.
            if (prihlasen)
            {
                //JO 16.9.2020 Zapoznámkováno do další verze
                //-------------------------

                string paramDvoufaktor = "0";

                //JH 30.3.2022 Kontrola rozšíření
                if (Globals.RAActive("TOTP_AUTHENTICATOR_PORTAL") || Zakaznik.IsCurrent("BLANKYT") || Zakaznik.IsCurrent("KS-PROGRAM"))
                {
                    paramDvoufaktor = Parametry.ParametrCti("DvoufaktorovyLogin", "0").ToString().Trim();
                }

                // JO 7.9.2021 - pokud se využívá SSO tak ignoruji tento parametr
                if (SpravaUziv.LoginType != SpravaUziv.LoginTypes.Form)
                {
                    paramDvoufaktor = "0";
                }

                if (paramDvoufaktor == "0") //standardní ověření (2-fakt. ov. není zapnuto)
                {
                    LogonLogger.WriteLine(login, "OK");
                    SpravaUziv.RedirectFromLoginPage(login); //MD 26.8.2016
                }                
                else if (paramDvoufaktor != "0") //2-fakt. ověření je zapnuto 
                {
                    //a uživatel prošel 1. krokem ověření (jméno a heslo)
                    if (ViewState["ControlValueLogin"] == null) ViewState["ControlValueLogin"] = "0";
                    if (ViewState["ControlValueLogin"].ToString() == "0" && SpravaUziv.LoginType == SpravaUziv.LoginTypes.Form && !string.IsNullOrEmpty(Globals.User))
                        TwoFactorChoose(securityCode, e);
                    else if (ViewState["ControlValueLogin"].ToString() == "1") //uživatel prošel 2. krokem ověření
                    {
                        LogonLogger.WriteLine(login, "OK");
                        ViewState["ControlValueLogin"] = "0";
                        SpravaUziv.RedirectFromLoginPage(login); //MD 26.8.2016
                    }
                }                
            }

            KSTrace.WriteLine("LoginButton_Click end");
        }

        protected void ChangePasswordButton_Click(object sender, EventArgs e)
        {
            string dbName = ddl_databaze.SelectedItem.Text;
            if (dbName == null) dbName = string.Empty;
            QueryStringCrypto qc = new QueryStringCrypto();
            qc.Redirect("~/Public/ChangePassword.aspx", "DBName=" + dbName);
        }

        //JH 11.10.2021 přesměrování podle parametru dvoufázového ověření
        protected void TwoFactorChoose(object sender, EventArgs e)
        {
            string paramDvoufaktor = Parametry.ParametrCti("DvoufaktorovyLogin").ToString().Trim();

            switch (paramDvoufaktor)
            {
                case "1":
                    SecurityCode_SendMail(sender, e);
                    break;
                case "2":
                    SecurityCode_SendSMS(sender, e);
                    break;
                case "3":
                    LogWithAuthenticator(sender, e);
                    break;
                case "4":
                    loginTable.Visible = false;
                    twoFactor.Visible = true;
                    if (String.IsNullOrEmpty(Parametry.ParametrCti("ApplicationIDSMS").Trim()))
                    {
                        SMSButton.Visible = false;
                    }
                    break;
            }
        }

        //JH 10.11.2022 - vybrane znaky kvuli sms
        protected string getHesloBezVybranychZnaku()
        {
            string abeceda = "ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnpqrstuvwxyz23456789";

            var kombinace = new char[6];
            var random = new Random();

            for (int i = 0; i < kombinace.Length; i++)
            {
                kombinace[i] = abeceda[random.Next(abeceda.Length)];
            }

            string output = new String(kombinace);

            return output;
        }

        //JH 8.10.2021 rozdělení původní metody pro zaslání kódu na 2
        //jedna samostatně pro mail a druhá pro SMS
        protected void SecurityCode_SendMail(object sender, EventArgs e)
        {
            loginTable.Visible = false;
            mail_Sms.Visible = true;
            twoFactor.Visible = false;

            ScriptManager.RegisterStartupScript(Page, Page.GetType(), "timeoutLogin", "timeout();", true);

            int dvoufaktorTimeout = KSProgram.Funkce.AppConfiguration.GetAppSetting<int>("DvoufaktorTimeout", 3);
            ScriptManager.RegisterStartupScript(Page, Page.GetType(), "timeoutTimeSet", "setTimeoutTime(" + dvoufaktorTimeout + ");", true);
            platnostKodu.Text = ReadGlobalResource.ZmenaTextu("[%Platnost kódu je%]") + ": " + dvoufaktorTimeout.ToString() + " min";

            string paramDvoufaktor = Parametry.ParametrCti("DvoufaktorovyLogin").ToString().Trim();

            if (paramDvoufaktor == "1" || paramDvoufaktor == "2" || paramDvoufaktor == "4")
            {
                //odešli kód a ulož ho někam na serveru
                string retezec = getHesloBezVybranychZnaku();
                string hash = HashAndCrypto.GetHash(retezec + "KSHASHVAL", HashAndCrypto.HashType.SHA512);
                hashVal.Value = hash;
                try
                {
                    decimal osCislo = Globals.OsCislo;
                    Globals.ClearRadPrava();
                    SpravaUziv.Odhlaseni();

                    //string email = KSProgram.Funkce.Pracovnik.GetEmailDoPrace(osCislo);
                    string email = KSProgram.Funkce.Pracovnik.GetEmailDoPrace(osCislo);
                    if (string.IsNullOrEmpty(email))
                    {
                        FailureLabel2.Text = ReadGlobalResource.ZmenaTextu("[%Email do práce není vyplněn%]");
                        return;
                    }

                    string subject = ReadGlobalResource.ZmenaTextu("[%Bezpečnostní kód%]");
                    TMailBody bodys = new TMailBody("SecurityKod", Globals.OsCisloAktUser);

                    // string body = TMailBody.SecurityCodeEmailBody(retezec);
                    string body = bodys.SecurityCodeEmailBody(retezec);
                    TMailMessage mail = new TMailMessage(email, subject, body, 0);
                    mail.Send();


                }
                catch (Exception ex)
                {
                    throw new Exception(ex.Message);
                }

            }
            else
            {
                LoginButton_Click(sender, e);
            }
        }

        //JH 8.10.2021 - zaslání ověřovacího kódu přes SMS
        protected void SecurityCode_SendSMS(object sender, EventArgs e)
        {
            loginTable.Visible = false;
            mail_Sms.Visible = true;
            twoFactor.Visible = false;

            ScriptManager.RegisterStartupScript(Page, Page.GetType(), "timeoutLogin", "timeout();", true);
            ServicePointManager.SecurityProtocol = SecurityProtocolType.Tls12;
            int dvoufaktorTimeout = KSProgram.Funkce.AppConfiguration.GetAppSetting<int>("DvoufaktorTimeout", 3);
            ScriptManager.RegisterStartupScript(Page, Page.GetType(), "timeoutTimeSet", "setTimeoutTime(" + dvoufaktorTimeout + ");", true);
            platnostKodu.Text = ReadGlobalResource.ZmenaTextu("[%Platnost kódu je%]") + ": " + dvoufaktorTimeout.ToString() + " min";

            string paramDvoufaktor = Parametry.ParametrCti("DvoufaktorovyLogin").ToString().Trim();

            if (paramDvoufaktor == "1" || paramDvoufaktor == "2" || paramDvoufaktor == "4")
            {
                //odešli kód a ulož ho někam na serveru
                string retezec = getHesloBezVybranychZnaku();
                string hash = HashAndCrypto.GetHash(retezec + "KSHASHVAL", HashAndCrypto.HashType.SHA512);
                hashVal.Value = hash;
                try
                {
                    decimal osCislo = Globals.OsCislo;
                    Globals.ClearRadPrava();
                    SpravaUziv.Odhlaseni();

                    KSProgram.SMSConnector.KSSmsConnector.Send(osCislo, "Bezpecnosti kod pro prihlaseni do KS Portalu: " + retezec);

                }
                catch (Exception ex)
                {
                    if (KSProgram.Funkce.AppConfiguration.GetAppSetting<string>("SecurityQRKod", "a") == "a" && !Globals.IsDevelopMode)
                    {
                        throw ex;
                    }
                    else
                    {
                        string title;
                        MessageBox.Alert(this.Page, ex.HtmlEncode(false, out title), title);
                    }
                }
            }
            else
            {
                LoginButton_Click(sender, e);
            }
        }

        //JH 8.10.2021 tlačítko s kontrolou ověřovacího kódu
        protected void ButtonSecurityCode_Click(object sender, EventArgs e)
        {

            string kod = securityCode.Text.Trim();
            if (HashAndCrypto.VerifyHash(kod + "KSHASHVAL", hashVal.Value.Trim(), HashAndCrypto.HashType.SHA512))
            {
                hashVal.Value = "";
                loginTable.Visible = true;
                mail_Sms.Visible = false;
                ViewState["ControlValueLogin"] = "1";
                LoginButton_Click(sender, e);
            }
            else
            {
                FailureLabel2.Text = ReadGlobalResource.ZmenaTextu("[%Zadaný kód není správny nebo vypršela jeho platnost%]");
            }


        }

        protected void LinkButton1_Click(object sender, EventArgs e)
        {
            loginTable.Visible = true;
            mail_Sms.Visible = false;
            twoFactor.Visible = false;
            Authenticator.Visible = false;
        }

        protected void LogWithAuthenticator(object sender, EventArgs e)
        {
            loginTable.Visible = false;
            mail_Sms.Visible = false;
            twoFactor.Visible = false;
            Authenticator.Visible = true;

            if (!String.IsNullOrEmpty(Pracovnik.GetEmailDoPrace(Globals.OsCislo)))
            {
                AuthenticationPrihlaseni.Visible = true;
                AuthenticationPrihlaseniFail.Visible = false;
            }
            else
            {
                AuthenticationPrihlaseni.Visible = false;
                AuthenticationPrihlaseniFail.Visible = true;
            }

            AuthenticatorMailSent.Visible = false;
            SendMailAuthenticator.Visible = true;
        }

        //JH 8.10.2021 ověření kódu z authentikátoru
        protected void ButtonAuthenticatorValidateClick(object sender, EventArgs e)
        {
            if (HashAndCrypto.HesloGet(Globals.OsCisloAktUser, 91) != null)
            {

                string pin = AuthenticatorPin.Text.Trim();
                if (KSProgram.Funkce.Authenticator.ValidateTwoFactorPIN(pin))
                {
                    loginTable.Visible = true;
                    mail_Sms.Visible = false;
                    twoFactor.Visible = false;
                    Authenticator.Visible = false;

                    ViewState["ControlValueLogin"] = "1";
                    LoginButton_Click(sender, e);
                }
                else
                {
                    Label2.Text = ReadGlobalResource.ZmenaTextu("[%Zadaný kód není správný%]");
                }

            }
            else
            {
                Label2.Text = ReadGlobalResource.ZmenaTextu("[%Vygenerujte si prosím nový authentikátor%]");
            }
        }

        protected void AuthenticatorCodeSend(object sender, EventArgs e)
        {
            KSProgram.Funkce.Authenticator.GenerateTwoFactorAuthentication();
            SendMailAuthenticator.Visible = false;
            AuthenticatorMailSent.Visible = true;
            LinkButton4.Visible = false;
            LinkButton3.Visible = false;
            AuthenticatorPin.Visible = false;
        }
    }
}