﻿using KSProgram.Data;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Web;
using System.Web.UI;
using static KSProgram.InfoBlob.InfoBlob;
using System.Drawing;
using System.Drawing.Imaging;
using System.Text;
using KSProgram;

namespace KSPortal.Obecne
{
    public partial class Blob : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            Response.Clear();

            if (!String.IsNullOrEmpty(Request.QueryString["p"]))
            {
                BlobResourceParameters pars = DecodeBlobUrl(Request.QueryString["p"]?.ToString());
                InfoBlobRadek blobradek;
                if (pars != null &&
                    (string.IsNullOrEmpty(pars.SessionId) ? true : pars.SessionId == System.Web.HttpContext.Current.Session.SessionID))
                {
                    byte[] blob = null;
                    if (pars.id.HasValue && Read(pars.id.Value, out blobradek))
                    {
                        blob = blobradek.informace;
                        if (pars.typ == BlobResourceType.Text)
                        {
                            byte[] bytes = Encoding.UTF8.GetBytes(Encoding.GetEncoding(blobradek.kodovani ?? 1200).GetString(blob));
                            Response.ContentType = "html/text";
                            Response.OutputStream.Write(bytes, 0, bytes.Length);
                            return;
                        }
                    }
                    else if (pars.os_cislo.HasValue)
                    {
                        if (Globals.IsDbConected) //PR 15.04.2021 
                        {
                            ParametryPrikazu pr = new ParametryPrikazu() { new KSParameter("os_cislo", System.Data.DbType.Decimal, pars.os_cislo) };
                            blob = KSProgram.Data.SqlPrikaz.ExecuteScalarBlob("select foto from fotografie where os_cislo = @os_cislo", ref pr);
                        }
                        if((blob?.Length ?? 0) == 0)
                        {
                            blob = File.ReadAllBytes(System.Web.HttpRuntime.AppDomainAppPath + "ImagesN\\user.png");
                        }
                    }
                    if (blob != null && blob.Length > 0)
                    {
                        if (pars.typ == BlobResourceType.Image)
                        {
                            bool isValid = Telerik.Web.UI.BinaryImageFormatHelper.IsJpeg(blob)
                                || Telerik.Web.UI.BinaryImageFormatHelper.IsPng(blob)
                                || Telerik.Web.UI.BinaryImageFormatHelper.IsGif(blob)
                                || Telerik.Web.UI.BinaryImageFormatHelper.IsBmp(blob)
                                || Telerik.Web.UI.BinaryImageFormatHelper.IsTiff(blob);
                            Image image;
                            if (!isValid)
                            {
                                image = Image.FromFile(System.Web.HttpRuntime.AppDomainAppPath + "ImagesN\\noImage.png");
                            }
                            else
                            {
                                MemoryStream stream = new MemoryStream(blob);
                                image = Image.FromStream(stream);
                            }

                            //resize
                            //------------------------
                            Size newSize = new Size();
                            if (!String.IsNullOrEmpty(Request.QueryString["maxWidth"]))
                            {
                                newSize.Width = int.Parse(Request.QueryString["maxWidth"]);
                            }
                            else if (pars?.MaxWidth != null)
                            {
                                newSize.Width = pars?.MaxWidth ?? 0;
                            }
                            if (!String.IsNullOrEmpty(Request.QueryString["maxHeight"]))
                            {
                                newSize.Height = int.Parse(Request.QueryString["maxHeight"]);
                            }
                            else if (pars?.MaxHeight != null)
                            {
                                newSize.Height = pars?.MaxHeight ?? 0;
                            }
                            if (!newSize.IsEmpty)
                            {
                                image = Resize(image, newSize);
                            }


                            if (!String.IsNullOrEmpty(Request.QueryString["minWidth"]))
                            {
                                int min = int.Parse(Request.QueryString["minWidth"]);
                                newSize.Width = newSize.Width < min ? min : newSize.Width;
                            }
                            else if (pars?.MinWidth != null)
                            {
                                newSize.Width = newSize.Width < pars.MinWidth.Value ? pars.MinWidth.Value : newSize.Width;
                            }
                            if (!String.IsNullOrEmpty(Request.QueryString["minHeight"]))
                            {
                                int min = int.Parse(Request.QueryString["minHeight"]);
                                newSize.Height = newSize.Height < min ? min : newSize.Height;
                            }
                            else if (pars?.MinHeight != null)
                            {
                                newSize.Height = newSize.Height < pars.MinHeight.Value ? pars.MinHeight.Value : newSize.Height;
                            }
                            if (!newSize.IsEmpty)
                            {
                                image = Resize(image, newSize);
                            }

                            //------------------------

                            Response.ContentType = "image/png";
                            image.Save(Response.OutputStream, ImageFormat.Png);
                        }
                    }
                }
            }
        }
        private Image Resize(Image image, Size newSize)
        {
            if (newSize == image.Size) return image;

            float rationW = (newSize.Width > 0) ? ((float)newSize.Width / (float)image.Size.Width) : 1;
            float ratioH = (newSize.Height > 0) ? ((float)newSize.Height / (float)image.Size.Height) : 1;
            float ratio = (rationW < ratioH) ? rationW : ratioH;
            if (ratio == 1) return image;

            int newWidth = (int)(image.Size.Width * ratio);
            int newHeight = (int)(image.Size.Height * ratio);

            Bitmap newImage = new Bitmap(newWidth, newHeight);
            using (Graphics gr = Graphics.FromImage(newImage))
            {
                gr.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.HighQualityBicubic;
                gr.CompositingQuality = System.Drawing.Drawing2D.CompositingQuality.HighQuality;
                gr.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
                gr.PixelOffsetMode = System.Drawing.Drawing2D.PixelOffsetMode.HighQuality;
                gr.DrawImage(image, new Rectangle(0, 0, newWidth, newHeight));
            }
            return (Image)newImage;
        }
    }
}