﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Reflection;
using KSProgram;
using KSProgram.Data;
using KSProgram.Prava;
using KSProgram.Obecne;
using System.Collections.Generic;
using System.Text.RegularExpressions;
using System.Globalization;
using KSProgram.Personal;
using KSProgram.Extensions;
using System.Linq;

public partial class Obecne_OAplikaci : TPage
{
    /// <summary>
    /// 
    /// </summary>
    protected override void OnInit(EventArgs e)
    {
        base.OnInit(e);
        EnableVisibleSeznamZam = false;
    }


    protected override void Page_Load(object sender, EventArgs e)
    {
        base.Page_Load(sender, e);
        if (IsPostBack) return;

        //nastavení aplikace
        this.verze.Text = Globals.Verze;
        this.uzivatel.Text = Globals.User;
        this.principal.Text = HttpContext.Current.User.ToString();
        this.roleProvider.Text = Roles.ApplicationName;

        string[] roles = PravaAkce.GetUserRoleNames(Globals.User);
        this.role.Text = string.Join(", ", roles);

        this.TypPravUzivatel.Text = Globals.RadkovePrava.TypPrav.ToString(); //PR 11.12.2019

        this.databaze.Text = Globals.DBName;
        this.databaze_rezim.Text = Globals.DbJeOstra ? KSProgram.WebControls.WebControlHelper.ZmenaTextu("[%ostrá%]") : KSProgram.WebControls.WebControlHelper.ZmenaTextu("[%testovací%]");
        ConnectionStringSettingsCollection connectionStrings = DatabaseInfo.GetConnectionStrings(); //MD 17.10.2013
        string provider = "";
        foreach (ConnectionStringSettings set in connectionStrings)
        {
            if (set.Name == Globals.DBName)
            {
                provider = set.ProviderName;
                break;
            }
        }
        this.poskytovatel.Text = provider;

        this.debugger.Text = System.Diagnostics.Debugger.IsAttached.ToString();
        
        //PR 21.08.2018 
        this.shortDateFormat.Text = CultureInfo.CurrentCulture.DateTimeFormat.ShortDatePattern; //pokud je 1. M, tak portál nebude fungovat správně, tam, kde je nastaven DateInput.DateFormat="d.M.yyyy"
        this.decimalSeparator.Text = CultureInfo.CurrentCulture.NumberFormat.NumberDecimalSeparator;

        //PH 30.5.2022 HID:61317
        this.Label_poradi_tydnu.Text = System.Globalization.CultureInfo.CurrentCulture.DateTimeFormat.CalendarWeekRule.ToString();

        //PR 19.01.2021 zákazník a aktivní rozšíření
        this.zakaznik.Text = Zakaznik.GetCustomer() + " (IČO " + Globals.ICO + (" " + Globals.ICO_EXT).TrimEnd() + ")";

        string aktivModRa = "";
        string[] moduly = Enum.GetNames(typeof(AppModul));
        foreach (string modul in moduly)
        {
            if (Zakaznik.HasModul(modul))
            {
                if (aktivModRa.Length > 0) aktivModRa += ", ";
                aktivModRa += modul;
            }
        }
        
        RA_keys RA = Globals.RA;        
        string[] test = {"BEZPAPIROVE_HR", "DANOVKA", "DCH_POTLACOVANI_PRESTAVEK", "DCH_CASTKA_VISIBLE", "DCH_SROVNAT_FOND_ENABLE", "EXTERNI_PRIHLASOVANI", 
                        "GDPR", "IMPORT_PRUCHODU", "KATALOG_VZDELAVACICH_AKTIVIT", "PIN_PRO_VYPLATNI_LISTKY", "ROZKONTACE PODLE TRIVI",
                        "SQL_GENERATOR", "SC UJETE KM", "VYVESKA", "VZD_PRESENCE", "WEBAPI_READER", "WEBAPI_WRITER"};
        foreach (string a in test)
        {
            if (RA.IsRAActiveInDB(a))
            {
                if (aktivModRa.Length > 0) aktivModRa += ", ";
                aktivModRa += a;
            }
            else if (RA.IsRAActiveInCode(a)) //rozšiření povolené v kódu budou mít nakonci vykřičník
            {
                if (aktivModRa.Length > 0) aktivModRa += ", ";
                aktivModRa += a + "!";
            }
        }
        //aktivni += " (Celkem v DB: " + RA.ActivCountInDB + ")" - 1 rozšíření vygeneruje až 3 záznamy, takže to by byla zavádějící informace;
        this.aktivModRA.Text = aktivModRa;

        P_detaily.Visible = (DbPlatforma.GetPlatformaString() == "OracleClient" ? true : false);

        //MiKo: načtu sestavu poměry, naleznu nejhlavnější poměr, vezmu hodnoty ze sloupcu zpracov_mzdy a zpracov_pers a zobrazím, 
        var hlPomer = Globals.GetOsCisloPomer(Globals.OsCisloAktUserNeboZastupovany, -1);//nejhlavnejsi pomer
        var zprac =
            new PracovniPomery(hlPomer.OsCislo)
            .Pomery()
            .ToList(new { cislo_pom = (short)0, zpracov_mzdy = "", zpracov_pers = "" })//předpis pro mapování dataTable to List
            .FirstOrDefault(x => x.cislo_pom.Equals(hlPomer.CisloPom));
        L_zprac_mzd.Text = zprac?.zpracov_mzdy ?? string.Empty;
        L_zprac_pers.Text = zprac?.zpracov_pers ?? string.Empty;
    }

    protected void LinkButtonDetaily_Command(object sender, CommandEventArgs e)
    {
        Table ctrl = (Table)this.FindControlRecursive("table_detaily");
        _naplnInfo(ctrl);
        ctrl.Visible = true;
        ((WebControl)sender).Visible = false;

    }

    protected void LinkButtonDetailySkryt_Command(object sender, CommandEventArgs e)
    {
        Table ctrl = (Table)this.FindControlRecursive("table_detaily");
        ctrl.Visible = false;

        WebControl c = (WebControl)this.FindControlRecursive("LinkButtonDetaily");
        c.Visible = true;
    }

    private void _naplnInfo(Table tab)
    {
        string pl = DbPlatforma.GetPlatformaString();
        if (pl == "SqlClient")
        {
        }
        else if (pl == "OracleClient")
        {
            using (IDataReader reader = SqlPrikaz.ExecuteReader("select parameter, value  from nls_session_parameters"))
            {
                int i = 0;
                while (reader.Read())
                {
                    string par = reader.GetString(0);
                    string val = reader.GetString(1);

                    TableRow row = new TableRow();
                    tab.Rows.AddAt(i, row);
                    TableCell cell_par = new TableCell();
                    TableCell cell_val = new TableCell();
                    row.Cells.Add(cell_par);
                    row.Cells.Add(cell_val);
                    cell_par.Text = par;
                    cell_val.Text = val;
                }
                reader.Close();
            }
        }
    }
}
