﻿using System;
using System.Data;
using System.Configuration;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;
using Telerik.Web.UI;
using KSProgram.WebControls;


/// <summary>
/// Zobrazuje JavaScriptové zprávy por uživatele
/// </summary>
public class MessageBox
{
    private static Int32 _defaultWidth = 400;
    private static Int32 _defaultHeight = 150;
    #region Public methods
    /// <summary>
    /// Zobrazuje zprávu
    /// </summary>
    /// <param name="message"></param>
    public static void Show(Control ctrl, string message)
    {
        ScriptManager.RegisterStartupScript(ctrl, typeof(Page), "MessageBox", "alert('" + message + "');", true);
    }
    /// <summary>
    /// Zobrazuje oznamovací dialog
    /// </summary>
    /// <param name="page">Stránka, která bude zobrazovat dialog</param>
    /// <param name="message">Zpráva (nebo HTML kód), která se má zobrazit</param>
    public static void Alert(Page page, string message)
    {
        //RadWindowManager mng = page.Master.FindControl("RadWindowManager1") as RadWindowManager;
        //if (mng != null)
        //{
        //    mng.RadAlert(message, _defaultWidth, _defaultHeight, "", "");
        //}
        //else
        //{
        //    if (message == null) message = "null";
        //    string script = string.Format("jAlert('{0}');", message);
        //    ScriptManager.RegisterStartupScript(page, typeof(Page), "jAlert", script, true);
        //}
        //PR 06.04.2022 chybí zde ošetření nepovolených znaků, zakomentovávám a volám přetíženou metodu
        string title = WebControlHelper.ZmenaTextu("[%Oznámení%]");
        Alert(page, message, title, null, null, "");
    }
    /// <summary>
    /// Zobrazuje oznamovací dialog
    /// </summary>
    /// <param name="page">Stránka, která bude zobrazovat dialog</param>
    /// <param name="message">Zpráva (nebo HTML kód), která se má zobrazit</param>
    /// <param name="title">Titulek zprávy</param>
    public static void Alert(Page page, string message, string title)
    {
        Alert(page, message, title, null, null, "");
    }
    /// <summary>
    /// Zobrazuje oznamovací dialog
    /// </summary>
    /// <param name="page">Stránka, která bude zobrazovat dialog</param>
    /// <param name="message">Zpráva (nebo HTML kód), která se má zobrazit</param>
    /// <param name="title">Titulek zprávy</param>
    /// <param name="width">šířka okna nebo null pro default</param>
    /// <param name="height">výška okna nebo null pro default</param>
    /// <param name="callBackFnName">Název JavaScript funkce, která bude volána při zavření okna nebo null</param>
    public static void Alert(Page page, string message, string title, Int32? width, Int32? height, string callBackFnName)
    {
        RadWindowManager mng = page.Master.FindControl("RadWindowManager1") as RadWindowManager;
        width = width == null ? _defaultWidth : width;
        height = height == null ? _defaultHeight : height;
        callBackFnName = callBackFnName == null ? "" : callBackFnName.Trim();
        // ošetření nepovolených znaků
        title = string.IsNullOrEmpty(title) ? title : title.Replace("\r\n", "<br />").Replace("\n", "<br />").Replace("\r", "<br />").Replace("\t", " ").Replace("`", "\"").Replace("'", "\"").Replace("\\", "\\\\");
        message = string.IsNullOrEmpty(message) ? message : message.Replace("\r\n", "<br />").Replace("\n", "<br />").Replace("\r", "<br />").Replace("\t", " ").Replace("`", "\"").Replace("'", "\"").Replace("\\", "\\\\");

        if (mng != null)
        {
            mng.RadAlert(message, width, height, title, callBackFnName);
        }
        else
        {
            if (message == null) message = "null";
            string script = string.Format("jAlert('{0}','{1}');", message, title);
            ScriptManager.RegisterStartupScript(page, typeof(Page), "jAlert", script, true);
        }
    }
    public static void Alert(Page page, Exception ex, Int32? width, Int32? height, string callBackFnName)
    {
        string title;
        Alert(page, KSProgram.ExceptionUtility.HtmlEncode(ex, out title), title, width == null ? 800 : width, height == null ? 500 : height, callBackFnName);
    }
    /// <summary>
    /// Zobrazuje potvrzovací dialog (ano/ne)
    /// </summary>
    /// <param name="page">Stránka, která bude zobrazovat dialog</param>
    /// <param name="message">Zpráva (nebo HTML kód), která se má zobrazit</param>
    /// <param name="title">Titulek zprávy</param>
    /// <param name="width">šířka okna nebo null pro default</param>
    /// <param name="height">výška okna nebo null pro default</param>
    /// <param name="callBackFnName">Název JavaScript funkce, která bude volána při zavření okna. V argumentu funkce (args) bude naplněn výsledek</param>
    public static void Confirm(Page page, string message, string title, Int32? width, Int32? height, string callBackFnName)
    {
        width = width == null ? _defaultWidth : width;
        height = height == null ? _defaultHeight : height;
        callBackFnName = callBackFnName == null ? "" : callBackFnName.Trim();
        RadWindowManager mng = page.Master.FindControl("RadWindowManager1") as RadWindowManager;
        if (mng != null)
        {
            mng.RadConfirm(message, callBackFnName, width, height, null, title);
        }
    }
    #endregion
}
