﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Configuration;
using KSProgram;
using KSProgram.Obecne;
using KSProgram.Funkce;
using System.Collections.Specialized;
using System.Security;
using Telerik.Web.UI;
using KSProgram.Data;

namespace KSPortal.Public
{
    public partial class RegistraceForm : TPrePage
    {
        bool rodneCisloKontrola = AppConfiguration.GetAppSetting<bool>("RegistraceRodneCislo", false);
        string emailInfo = AppConfiguration.GetAppSetting<string>("RegistraceUzivatele_EmailInfo", "Zadejte e-mail, který máte v kontaktech zaměstnance naveden jako 'E-mail do práce'"); // MZ 27.3.2018

        public RegistraceForm()
        {
        }

        protected override void Page_Load(object sender, EventArgs e)
        {
            string registrace = AppConfiguration.GetAppSetting<string>("Registrace", null);
            if (registrace != "user" || SpravaUziv.LoginType != SpravaUziv.LoginTypes.Form)
                throw new System.Security.SecurityException("Přístup byl zamítnut");

            //PR 24.10.2022 chtěl jsem tu dát následujují kontrolu, ale nakonec nedal. Někteří totiž možná registrují uživatele do agendy prtl_user jen kvůli tomu, 
            //              aby se správci login nabídl v přiřazování aplikačních rolí, seznam uživatelů se v portále totiž dynamicky nenačítá z AD, jak je tomu v desktopové aplikaci.
            //if (SpravaUziv.TypOvereni != SpravaUziv.TypOvereniEnum.KSPU && SpravaUziv.ZdrojOsCisla != SpravaUziv.ZdrojOsCislaType.KSPU)
            //    throw new System.Security.SecurityException("Přístup byl zamítnut, registrace je povolena pouze pro TypOvereni=kspu nebo ZdrojOsCisla=kspu (jinak nedává smysl)");

            base.Page_Load(sender, e);

            //specialita zatim pro phoenix2 je treba zapis ve webconfigu do appSettings
            /*< !--Pokud ma vice uzivatelu stejny email tak jestli musi krome os. cisla zadat i cislo rodne-- >
            <add key = "RegistraceRodneCislo" value = "true" />*/
            ZobrazRodneCislo(rodneCisloKontrola);

            //načtu seznam databázových připojení
            ConnectionStringSettingsCollection connectionStrings = DatabaseInfo.GetConnectionStrings();
            for (int i = 0; i < connectionStrings.Count; i++)
            {
                if (!connectionStrings[i].Name.Contains("hidden_"))
                {
                    DropDownListItem item = new DropDownListItem(connectionStrings[i].Name, connectionStrings[i].Name);
                    ddl_databaze.Items.Add(item);
                }
            }
            
            //_SetEmailInfo(); // MZ 6.2.2018 Doplnění vlastní nápovědy pro zadání e-mailu. // MZ 27.3.2018 Zapoznámkováno, nápověda se přebere z web.config.
            //RadToolTip1.Text = emailInfo; // MZ 27.3.2018 Doplnění vlastní nápovědy pro zadání e-mailu. // JH 10.2.2023 zakomentovano
        }

        protected void TLB_Registrace_Click(object sender, EventArgs e)
        {
            RL_RegReturn.Text = string.Empty;

            string dbName = ddl_databaze.SelectedValue;
            string login = RTB_Login.Text;
            string heslo = RTB_Heslo.Text;
            string hesloZnovu = RTB_HesloZnovu.Text;
            string email = RTB_Email.Text;
            decimal osCislo = Convert.ToDecimal(RTB_OsCislo.Value);

            if (string.IsNullOrEmpty(login))
            {
                L_RegReturnV.ErrorMessage = "[%Jméno je povinný údaj%]";
                L_RegReturnV.IsValid = false;
                return;
            }
            if (string.IsNullOrEmpty(heslo))
            {
                L_RegReturnV.ErrorMessage = "[%Heslo je povinný údaj%]";
                L_RegReturnV.IsValid = false;
                return;
            }
            if (string.IsNullOrEmpty(email))
            {
                L_RegReturnV.ErrorMessage = "[%Email je povinný údaj%]";
                L_RegReturnV.IsValid = false;
                return;
            }

            if (heslo != hesloZnovu)
            {
                L_RegReturnV.ErrorMessage = "[%Zadaná hesla nejsou shodná%]";
                L_RegReturnV.IsValid = false;
                return;
            }

            //if (AppConfiguration.GetAppSetting<string>("TypOvereni", null) == "ad")
            if (SpravaUziv.TypOvereni == SpravaUziv.TypOvereniEnum.AD)
            {
                //MD 17.2.2017 Kontrola, zda je uživatel platný v AD. (kvůli DATARTu)
                try
                {
                    Globals.ADPripojeni.ValidateUser(login, heslo);
                }
                catch (Exception exc)
                {
                    L_RegReturnV.ErrorMessage = exc.Message;
                    L_RegReturnV.IsValid = false;
                    return;
                }
            }

            List<string> messages = new List<string>();
            try
            {
                string rodCisl = null;
                if (rodneCisloKontrola)
                    rodCisl = RNTB_RodneCislo.Value == null ? null : RNTB_RodneCislo.Value.ToString();

                SpravaUziv.RegistrationWithEmail(dbName, login, heslo, email, osCislo, rodCisl);
                messages.Add(ReadGlobalResource.ZmenaTextu("[%Registrace proběhla v pořádku%]"));
                SpravaUziv.SendActivationEmail(dbName, email, osCislo);
                messages.Add(ReadGlobalResource.ZmenaTextu("[%Aktivační email byl odeslán%]"));
            }
            catch (SecurityException exc)
            {
                L_RegReturnV.ErrorMessage = exc.Message;
                L_RegReturnV.IsValid = false;
                return;
            }

            RL_RegReturn.Text = string.Join("<br />", messages);
        }


        private void ZobrazRodneCislo(bool visible)
        {
            RadLabel9.Visible = visible;
            RNTB_RodneCislo.Visible = visible;
        }

        // MZ 27.3.2018 Zapoznámkováno, nápověda se přebere z web.config.
        //// MZ 6.2.2018
        ///// <summary>
        ///// Doplnění vlastní nápovědy pro zadání e-mailu.
        ///// </summary>
        //private void _SetEmailInfo()
        //{
        //    string dbName = ddl_databaze.SelectedValue;
        //    Globals.Db = DatabasePodleParam.Create(dbName);

        //    string emailInfo = Parametry.ParametrCti("RegistraceUzivatele_EmailInfo");
        //    if (!string.IsNullOrEmpty(emailInfo)) RadToolTip1.Text = emailInfo;
        //}

        //protected void ddl_databaze_ItemSelected(object sender, DropDownListEventArgs e)
        //{
        //    _SetEmailInfo(); // Doplnění vlastní nápovědy pro zadání e-mailu.
        //}
    }
}