﻿using KSProgram;
using KSProgram.Uchazeci;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Data;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

namespace KSPortal.Public
{
    public partial class UchazecEvidenceSouhlas : TPrePage
    {
        private decimal _EvCislo;
        private DateTime _DatumSouhlasu;

        public UchazecEvidenceSouhlas()
        {
            DefaultDatabase = true; //PR 15.04.2021 
        }

        protected override void Page_Load(object sender, EventArgs e)
        {
            base.Page_Load(sender, e);

            QueryStringCrypto qc = new QueryStringCrypto();
            NameValueCollection coll = qc.DecryptRequestQuery();
            _EvCislo = Convert.ToDecimal(coll["evCislo"]);

            if (_EvCislo == 0) return;
            
            short delka101 = Uchazeci.GetDelkaSouhlasuSEvidenci();
            _DatumSouhlasu = DateTime.Today.AddMonths(delka101).Date;

            if (!IsPostBack)
            {
                string zakaznik = KSProgram.Zakaznik.GetNazev().Trim();

                string[] mesic = { "", "měsíc", "měsíce", "měsíce", "měsíce", "měsíců" };
                string mesicText = mesic[5];
                if (delka101 < 5) { mesicText = mesic[delka101]; }

                string info = "Stiskem tlačítka 'Odeslat' dojde k prodloužení souhlasu uchazeče se zpracováním osobních údajů v souladu se zákonem č. 101/2000 Sb. (zákon o ochraně osobních údajů)." +
                    " Údaje budou uloženy v databázi uchazečů o zaměstnání ve společnosti " + zakaznik +
                    ", a to po dobu " + delka101.ToString() + " " + mesicText + " ode dne vyjádření souhlasu." +
                    " Tyto údaje budou uchovávány a zpracovávány pouze za čelem případného budoucího výběru zaměstnance z řad uchazečů o zaměstnání.";

                L_SouhlasText.Text = info;
            }
        }

        protected void UchazecData_NeedDataSource(object sender, Telerik.Web.UI.RadDataFormNeedDataSourceEventArgs e)
        {
            if (_EvCislo == 0) return;

            DataTable data = null;
            
            KSProgram.Personal.Uchazec uchazec = new KSProgram.Personal.Uchazec();
            data = uchazec.OsobniUdaje(_EvCislo);

            if (data == null || data.Rows.Count == 0)
            {
                P_UchazecData.Visible = false;
                P_NoData.Visible = true;
            }

            if (data != null)
            {
                UchazecData.DataSource = data;
            }
        }

        protected void UchazecSouhlas_NeedDataSource(object sender, Telerik.Web.UI.RadDataFormNeedDataSourceEventArgs e)
        {
            ArrayList list = new ArrayList();
            list.Add(new DatSouhlItem(_DatumSouhlasu));

            UchazecSouhlas.DataSource = list;
        }

        private class DatSouhlItem
        {
            public DateTime dat_souhl101 { get; set; }
           
            public DatSouhlItem(DateTime aDate)
            {
                dat_souhl101 = aDate;
            }
        }

        protected void BTN_Odeslat_Click(object sender, EventArgs e)
        {
            try
            {
                Uchazec uchazec = new Uchazec(_EvCislo);
                int ret = uchazec.Update_SouhlasSEvidenci(_DatumSouhlasu);

                P_UchazecData.Visible = false;
                P_Odeslano.Visible = true;
            }
            catch
            {
                throw new KSProgram.Exceptions.NoticeException("Bohužel, prodloužení souhlasu s evidencí se nepodařilo uložit do databáze.");
            }
        }
    }
}