﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Services;
using System.Web.Services.Protocols;
using System.Xml.Serialization;
using KSProgram;
using System.Web.Security;
using System.Collections.Specialized;

namespace KSPortal.WebServices
{
    /// <summary>
    /// Summary description for BasicAuthService
    /// </summary>
    [WebService(Namespace = "http://ksprogram.cz/KSPortal/WebServices/BasicAuthService/")]
    [WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
    [System.ComponentModel.ToolboxItem(false)]
    // To allow this Web Service to be called from script, using ASP.NET AJAX, uncomment the following line. 
    // [System.Web.Script.Services.ScriptService]
    public class BasicAuthService : System.Web.Services.WebService
    {
        /// <summary>
        /// Authentication result
        /// </summary>
        public class AuthResult
        {
            public string Token;
            public string DbName;
            public string UserName;
            public string Password;
            public bool IsValid;
            public Exception Exception;
        }

        /// <summary>
        /// Create query string from collection of parameters
        /// </summary>
        /// <param name="coll">Collection of parameters</param>
        /// <returns>Query string</returns>
        private string CreateQueryString(NameValueCollection coll)
        {
            string queryString = string.Empty;

            List<string> pairs = new List<string>();
            foreach (string key in coll.AllKeys)
            {
                string pair = string.Format("{0}={1}", HttpUtility.UrlEncode(key), HttpUtility.UrlEncode(coll[key]));
                pairs.Add(pair);
            }

            if (pairs.Count > 0)
                queryString = "?" + string.Join("&", pairs.ToArray());
            return queryString;
        }

        /// <summary>
        /// Perform authentication
        /// </summary>
        /// <param name="userName">User name</param>
        /// <param name="password">Password</param>
        /// <param name="relativeUrl">Virtual path from root of web application for redirect after login. If value is null or empty string or wrong, virtual path is not set.</param>
        /// <returns>Login URL</returns>
        [WebMethod(EnableSession = true, Description="Perform user autentication and return login URL.")]
        public string AuthenticateUser(string userName, string password, string relativeUrl)
        {
            AuthResult result = new AuthResult();
            result.Token = Guid.NewGuid().ToString("N"); //Create the authenticated token
            result.UserName = userName;
            result.Password = password;
            result.IsValid = false;

            try
            {
                string dbName = KSProgram.Data.DatabasePodleParam.GetDefautlDbName();
                result.DbName = dbName;
                SpravaUziv.FormPrihlaseni(dbName, userName, password);
                result.IsValid = true;
            }
            catch (Exception exc)
            {
                result.Exception = exc;
            }

            //Store user information to cache
            HttpRuntime.Cache.Add(result.Token, result, null, System.Web.Caching.Cache.NoAbsoluteExpiration,
            TimeSpan.FromMinutes(Session.Timeout), System.Web.Caching.CacheItemPriority.NotRemovable, null);

            //Login URL
            string url = HttpContext.Current.Request.Url.Scheme
                + "://" + HttpContext.Current.Request.Url.Authority
                + HttpContext.Current.Request.ApplicationPath
                + "/Public/TokenLogin.aspx";

            //Query string
            NameValueCollection queryColl = new NameValueCollection();
            queryColl.Add("token", result.Token);

            if (!string.IsNullOrWhiteSpace(relativeUrl))
            {
                try
                {
                    string returnUrl = VirtualPathUtility.ToAbsolute(relativeUrl);
                    queryColl.Add("ReturnUrl", returnUrl);
                }
                catch
                { }
            }
            string queryString = CreateQueryString(queryColl);

            return url + queryString;
        }

        /// <summary>
        /// Validate user token
        /// </summary>
        /// <param name="token">User token</param>
        /// <returns>Authentication result</returns>
        public AuthResult ValidateToken(string token)
        {
            if (string.IsNullOrWhiteSpace(token))
                return null;

            AuthResult result = HttpRuntime.Cache[token] as AuthResult;
            HttpRuntime.Cache.Remove(token);
            return result;
        }
    }
}
