﻿using KSProgram;
using KSProgram.Data;
using KSProgram.Diagnostics;
using KSProgram.Dochazka;
using KSProgram.Funkce;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Services;
using System.Web.Services.Protocols;
using System.Xml.Serialization;
using KSProgram.Extensions;
using static KSProgram.Dochazka.Dch_Casy;

namespace KSPortal.WebServices
{
    /// <summary>
    /// Summary description for DchPruchody
    /// </summary>
    [WebService(Namespace = "ksprtl/WebServices")]
    [WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
    [System.ComponentModel.ToolboxItem(false)]
    // To allow this Web Service to be called from script, using ASP.NET AJAX, uncomment the following line. 
    // [System.Web.Script.Services.ScriptService]
    public class DchPruchody : System.Web.Services.WebService
    {

        public AuthKeyHeader AuthHeader;

        /// <summary>
        /// Proceed authentication
        /// </summary>
        /// <param name="auth">Authentication object</param>
        /// <returns>Error message</returns>
        private string Authentication(AuthKeyHeader auth)
        {
            //PH 23.6.2020 HID:51867
            if (!Globals.RAActive("IMPORT_PRUCHODU")) return "Autentizace:Není povoleno rozšíření Import průchodů. Kontaktujte obchodní oddělení firmy KS program";

            string configKey = KSProgram.Funkce.AppConfiguration.GetAppSetting<string>("WsAuthKey", null);

            if (configKey == null) return null;

            if (auth == null) return "Autentizace: metoda musí být volána s autentizační SOAP hlavičkou AuthKeyHeader.";
            if (configKey == null) return "Autentizace: není nastaven autentizační klíč AppSettings/WsAuthKey v konfiguračním souboru KS portálu.";
            if (auth.AuthKey != configKey)
                return "Autentizace: nesprávný klíč.";
            return null;
        }

        /// <summary>
        /// Import průchodu do docházky
        /// </summary>
        /// <param name="pr"></param>
        /// <returns></returns>
        [WebMethod(EnableSession = true, Description = "Import průchodů do KS docházky.")]
        [SoapHeader("AuthHeader")]
        public DchPruchodResult PruchodVloz(DchPruchod pr)
        {
            DateTime? ld_posl_pruchod;
            string res_out;
            string opak_pruchod;

            DchPruchodResult r = new DchPruchodResult();
            r.ErrorCode = -99;
            r.ErrorString = "Chyba";

            try
            {
                if (null == Globals.Db)
                {
                    Globals.Db = DatabasePodleParam.Create(null);
                    KSTrace.WriteLine("Aktivní DB", Globals.Db.ConnectionStringWithoutCredentials);
                }

                //AUTENTIZACE
                string authMessage = Authentication(AuthHeader);
                if (authMessage != null)
                {
                    r.ErrorCode = -401;
                    r.ErrorString = authMessage;
                    return r;
                }

                //samotné vkládání
                //----------------------------------------               
                if (pr.CisloPom==null)
                {
                    short ai_pomer;
                    Prac_pom.Platny_Pomer_Den(pr.OsCislo, DateTime.Now.Date, out ai_pomer);
                    if (ai_pomer < 0)
                    {
                        r.ErrorCode = -98;
                        r.ErrorString = "Poměr nezjištěn.";
                        return r;
                    }
                    pr.CisloPom = ai_pomer;
                }

                Str_Dch_Casy str = new Str_Dch_Casy();
                str.Os_cislo = pr.OsCislo;
                str.Cislo_pom = (short)pr.CisloPom;
                str.Cas = (DateTime)pr.Cas;
                str.Kod = pr.Kod;
                str.Operace = "t";
                str.Uzivatel = "WebService";
                str.Poznamka = pr.Poznamka;

                //PH 17.2.2017 v případě, že se přenáší odchod, kontroluju jestli předchozí průchod není také odchod
                //----------------------------------------   
                if (AppConfiguration.GetAppSetting<bool>("DchPruchody.PruchodVloz.DoplnOdchod", false))
                {
                    if (!PorizeniDen.PosledniIntevalovyPruchodOk(pr.OsCislo, (short)pr.CisloPom, pr.Kod, (DateTime)pr.Cas, out res_out, out opak_pruchod, out ld_posl_pruchod))
                    {
                        if ((ParametryDochazky.AUT_ODCHOD) && (opak_pruchod == "start"))
                            Dch_Casy.InsertKonecIntervalu(str);
                        else
                        {
                            r.ErrorCode = -97;
                            r.ErrorString = "Opakovaný odchod.";
                            return r;
                        }
                    }
                }
                //---------------------------------------- 
                
                ParametryDochazky pd = new ParametryDochazky();
                if (pd.IsKodOdchod(str.Kod))
                    Dch_Casy.InsertKonecIntervalu(str);
                else
                    Dch_Casy.DbInsert(str);
               
                //----------------------------------------
            }
            catch (Exception ex)
            {                
                r.ErrorString = ex.Message;
                if (null != ex.InnerException) r.ErrorString += ";\r\nInnerException=" + ex.InnerException.Message;
                if (null != ex.Source) r.ErrorString += ";\r\nSource=" + ex.Source;
                if (null != ex.StackTrace) r.ErrorString += ";\r\nStackTrace=" + ex.StackTrace;
                return r;
            }
            r.ErrorCode = 0;
            r.ErrorString = string.Empty;
            
            return r;
        }
    }

    /// <summary>
    /// Návratová typ volání služby
    /// </summary>
    public class DchPruchodResult
    {
        /// <summary>
        /// záporná hodnota = chyba, 0 = úspěch, kladná hodnota = warning
        /// </summary>
        public int ErrorCode;
        /// <summary>
        /// doplňující text
        /// </summary>
        public string ErrorString;
    }

    /// <summary>
    /// 
    /// </summary>
    public class DchPruchod
    {
        /// <summary>
        /// Osobní číslo
        /// </summary>
        [XmlElementAttribute(IsNullable = false)]
        public decimal OsCislo;

        /// <summary>
        /// Číslo poměru
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public short? CisloPom;

        /// <summary>
        /// Kód odpovídající našemu číselníku docházkových kódů
        /// </summary>
        [XmlElementAttribute(IsNullable = false)]
        public int Kod;

        /// <summary>
        /// Pracovní středisko, odpovídající našemu číselníku středisek
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public string Prac_str;

        /// <summary>
        /// Čas průchodu
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public DateTime? Cas;

        /// <summary>
        /// doba
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]       
        public string Doba;

        /// <summary>
        /// Kód prostředí, zatím se nevyužívá
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public string Kodprst;

        /// <summary>
        /// Sazba
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public decimal? Sazba;

        /// <summary>
        /// Částka
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public decimal? Castka;

        /// <summary>
        /// uživatelské pole
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public string C_pole1;

        /// <summary>
        /// uživatelské pole
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public string C_pole2;

        /// <summary>
        /// uživatelské pole
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public long? N_pole1;

        /// <summary>
        /// uživatelské pole
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public long? N_pole2;

        /// <summary>
        /// poznámka
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public string Poznamka;
    }
}
