﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Services;
using System.Web.Services.Protocols;
using System.Xml.Serialization;
using KSProgram.Data;
using System.Web.Script.Services;
using KSProgram;

namespace KSPortal.WebServices
{
    /// <summary>
    /// Summary description for ApplicationUsers
    /// </summary>
    [WebService(Namespace = "KSPortal/WebServices/PortalUsers/")]
    [WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
    [System.ComponentModel.ToolboxItem(false)]
    // To allow this Web Service to be called from script, using ASP.NET AJAX, uncomment the following line. 
    // [System.Web.Script.Services.ScriptService]
    public class PortalUsers : System.Web.Services.WebService
    {
        public AuthKeyHeader AuthHeader;

        /// <summary>
        /// Proceed authentication
        /// </summary>
        /// <param name="auth">Authentication object</param>
        /// <returns>Error message</returns>
        private string Authentication(AuthKeyHeader auth)
        {
            if (auth == null) return "Authentication is required.";
            string configKey = KSProgram.Funkce.AppConfiguration.GetAppSetting<string>("WsAuthKey", null);
            if (configKey == null) return "Authentication key in configuration is reguired.";
            if (auth.AuthKey != configKey)
                return "Authentication is not success.";
            return null;
        }
        /// <summary>
        /// User
        /// </summary>
        public class AppUser
        {
            public decimal OsCislo { get; set; }
            public string Login { get; set; }
            [XmlElement(IsNullable = true)]
            public string Password { get; set; }
            [XmlElement(IsNullable = true)]
            public DateTime? PasswordValidFrom { get; set; }
        }
        /// <summary>
        /// List of users
        /// </summary>
        public class AppUsers : List<AppUser> { }

        /// <summary>
        /// Import application users in XML data 
        /// </summary>
        /// <param name="Users">Application users</param>
        /// <returns>Result</returns>
        [WebMethod(EnableSession = true)]
        [SoapHeader("AuthHeader")]
        public string ImportXmlData(AppUsers Users)
        {
            string authMessage = Authentication(AuthHeader);
            if (authMessage != null) return authMessage;

            foreach (AppUser user in Users)
            {
                //MD 13.4.2015 Kontrola, zda není login již použit u jiného uživatele (zaměstnance).
                KSProgram.Globals.Db = KSProgram.Data.DatabasePodleParam.Create(null); //MD 26.5.2017 Je potřeba připojit databázi
                string testSql = "select os_cislo from prtl_users where login = " + SqlPrikaz.ValueToString(user.Login) + " and os_cislo <> " + user.OsCislo;
                decimal oldOsCislo = SqlPrikaz.ExecuteScalarDecimal(testSql, 0);
                if (oldOsCislo > 0)
                {
                    return "Login " + user.Login +" je již použit u osobního čísla " + oldOsCislo.ToString();
                }

                SqlPrikaz.ExecuteNonQuery("delete from prtl_users where os_cislo = " + SqlPrikaz.ValueToString(user.OsCislo));

                // TP 6.11.2017 nově se heslo hashuje
                //----------------------------------------------------------------------------------------------------
                //string heslo = KSProgram.HashAndCrypto.CodePassword(user.Password, 44);
                string heslo = KSProgram.HashAndCrypto.GetHash(user.Password, HashAndCrypto.HashType.SHA1, 44);
                //----------------------------------------------------------------------------------------------------

                string insertSql = "insert into prtl_users (os_cislo, login, heslo, heslo_od)" +
                    " values (" + SqlPrikaz.ValueToString(user.OsCislo) +
                    ", " + SqlPrikaz.ValueToString(user.Login) +
                    ", " + SqlPrikaz.ValueToString(heslo) +
                    ", " + SqlPrikaz.ValueToString(user.PasswordValidFrom) + ")";
                SqlPrikaz.ExecuteNonQuery(insertSql);
            }

            return "OK";
        }
    }
}
