﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Services;
using System.Xml.Serialization;
using KSProgram;
using KSProgram.Sluzebky;
using KSProgram.Pozadavky;
using KSProgram.Funkce;
using KSProgram.Data;
using System.Data.Common;
using KSProgram.Security;
using System.IO;
using KSProgram.Diagnostics;
using KSProgram.Obecne;
using System.Web.Security;
using System.Web.Services.Protocols;
using System.Net.Mail;

namespace KSPortal.WebServices
{
    /// <summary>
    /// Summary description for SCesty2
    /// </summary>
    [WebService(Namespace = "ksprtl/WebServices")]
    [WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
    [System.ComponentModel.ToolboxItem(false)]
    // To allow this Web Service to be called from script, using ASP.NET AJAX, uncomment the following line. 
    // [System.Web.Script.Services.ScriptService]
    public class SCesty2 : System.Web.Services.WebService
    {
        public AuthKeyHeader AuthHeader;

        /// <summary>
        /// Proceed authentication
        /// </summary>
        /// <param name="auth">Authentication object</param>
        /// <returns>Error message</returns>
        private string Authentication(AuthKeyHeader auth)
        {
            string configKey = KSProgram.Funkce.AppConfiguration.GetAppSetting<string>("WsAuthKey", null);

            //if (Zakaznik.IsCurrent("VARROC") && auth == null && configKey == null) return null;

            if (auth == null) return "Autentizace: metoda musí být volána s autentizační SOAP hlavičkou AuthKeyHeader.";
            if (configKey == null) return "Autentizace: není nastaven autentizační klíč AppSettings/WsAuthKey v konfiguračním souboru KS portálu.";
            if (auth.AuthKey != configKey)
                return "Autentizace: nesprávný klíč.";
            return null;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="cesta">Třída s daty služební cesty včetně podčástí</param>
        /// <returns></returns>
        [WebMethod(EnableSession = true, Description = "Import nebo aktualizace služební cesty.")]
        [SoapHeader("AuthHeader")]
        public SCResult CestaVloz2(SCCesta2 c)
        {
            /*this.Context.Request.InputStream.Position = 0;
            StreamReader stream = new StreamReader(this.Context.Request.InputStream);
            string x = stream.ReadToEnd();
            string xml = HttpUtility.UrlDecode(x);
            xml += "";*/

            SCResult r = new SCResult();
            r.ErrorCode = -99;
            r.ErrorString = "Chyba";

            bool lb_CisloNaVstupu = false;

            try
            {
                //r.ErrorString = r.ErrorString.Substring(255); //test chyby

                //PRIPOJENI K DB
                if (null == Globals.Db)
                {
                    Globals.Db = DatabasePodleParam.Create(null);
                    KSTrace.WriteLine("Aktivní DB", Globals.Db.ConnectionStringWithoutCredentials);
                }

                //AUTENTIZACE
                string authMessage = Authentication(AuthHeader);
                if (authMessage != null)
                {
                    r.ErrorCode = -401;
                    r.ErrorString = authMessage;
                    return r;
                }

                //VALIDACE HODNOT
                r = CestaValidaceHodnot(ref c);
                if (r.ErrorCode < 0)
                    return r;
                r.ErrorCode = -2;

                lb_CisloNaVstupu = !string.IsNullOrEmpty(c.Cislo);

                Decimal os_cislo = OsobaNaOsCislo(c.Zamestnanec);
                String zdroj = "E";
                if (String.IsNullOrEmpty(c.PocMisto)) c.PocMisto = SlcHlavicka.VychoziMistoOdjezdu(os_cislo);
                if (String.IsNullOrEmpty(c.KonMisto)) c.KonMisto = c.PocMisto;
                
                String spolucestujici = "";
                decimal?[] spolu = new decimal?[10];
                if (c.Spolucestujici != null && c.Spolucestujici.Count > 0)
                {
                    for (int i = 0; i < Math.Min(10, c.Spolucestujici.Count); i++)
                    {
                        spolu[i] = OsobaNaOsCislo(c.Spolucestujici[i]);
                        if (spolucestujici.Length > 0) spolucestujici += ", ";
                        if (spolu[i] != null)
                            spolucestujici += SlcHlavicka.ZkraceneJmeno(spolu[i].Value);
                        else
                            spolucestujici += c.Spolucestujici[i].Login;
                    }
                    spolucestujici = Trunc(spolucestujici, 120);
                }

                //pokud je zadáno číslo, tak zjišťuji, zda SC existuje (update místo insertu)
                Int32 ev_cislo = 0;
                bool pouzitWorkFlow = (c.WorkFlow != null && c.WorkFlow.Count > 0);
                String tabulka = pouzitWorkFlow ? "workfl_sluz_cesty" : "sluz_cesty";
                bool lb_update = false;
                if (!String.IsNullOrEmpty(c.Cislo))
                {
                    ev_cislo = SqlPrikaz.ExecuteScalarInt32("select ev_cislo from sluz_cesty where cislo='" + c.Cislo + "' and os_cislo=" + os_cislo, 0);
                    if (ev_cislo > 0)
                        tabulka = "sluz_cesty";
                    else
                    {
                        ev_cislo = SqlPrikaz.ExecuteScalarInt32("select ev_cislo from workfl_sluz_cesty where cislo='" + c.Cislo + "' and os_cislo=" + os_cislo, 0);
                        if (ev_cislo > 0)
                            tabulka = "workfl_sluz_cesty";
                    }

                    if (ev_cislo > 0)
                    {
                        lb_update = true;                        
                    }
                }
                //Pokud má být proveden insert zkontroluji kolizi termínu s jinou SC
                if (!lb_update)
                    if (!SlcFunkce.of_TerminVolny(os_cislo, ev_cislo, c.DatOd.Date, c.DatOd.ToString("HH:mm"), c.DatDo.Date, c.DatDo.ToString("HH:mm"), out r.ErrorString))
                    {
                        r.ErrorCode = -3;
                        r.ErrorString = r.ErrorString.Replace("[%", "").Replace("%]", "");
                        return r;
                    }                

                r.ErrorCode = -4;

                bool lb_vlastni, lb_sluzebni;
                string ls_spz, ls_phm = "", ls_pov_ruc = "", ls_druh = "";
                decimal ldc_vlastnik = 0, ldc_prum_spotreba = 0, ldc_spotreba_mesto = 0, ldc_spotreba_mimo = 0;
                //PR 10.11.2015 //decimal ldc_zakl_nahr = 0, ldc_cena_phm = 0;
                ls_spz = c.Spz;
                lb_vlastni = c.DruhDopr.Contains("AUV") || c.DruhDopr.Contains("NAV") || c.DruhDopr.Contains("MOV");
                lb_sluzebni = c.DruhDopr.Contains("AUS") || c.DruhDopr.Contains("NAS") || c.DruhDopr.Contains("MOS");
                //if (lb_vlastni) 
                //PR 9.11.2017 
                if (lb_vlastni || (lb_sluzebni && Parametr_cti.Get("VuzSluzSoukPHM", DateTime.Today, ViceIco.GetIdPodnikuZamestnance(os_cislo)) == "a"))
                {
                    KSProgram.Sluzebky.SlcVozidla.NactiInfoOvozidle(ls_spz, out ldc_vlastnik, out ls_druh, out ls_phm, out ldc_spotreba_mesto, out ldc_spotreba_mimo, out ldc_prum_spotreba, out ls_pov_ruc);
                    //ldc_zakl_nahr = KSProgram.Sluzebky.SlcSazby.of_zjistiSazbuZaklNahr(c.DatOd, ls_druh);
                    //ldc_cena_phm = KSProgram.Sluzebky.SlcCisPhm.Cena(ls_phm, c.DatDo);
                }

                //if (ldc_vlastnik > 0 && ldc_vlastnik != os_cislo)
                //PR 9.11.2017 
                if (ldc_vlastnik != os_cislo)
                {//PR 13.12.2012 pokud neni vlastnikem, tak to vse nastavim na null
                    ldc_spotreba_mesto = 0;
                    ldc_spotreba_mimo = 0;
                    ldc_prum_spotreba = 0;
                    ls_phm = "";
                    //ldc_cena_phm = 0;
                    //ldc_zakl_nahr = 0;
                }

                decimal ldc_povolil = OsobaNaOsCislo(c.Povolil);

                r.ErrorCode = -5;

                using (DbConnection connection = SqlPrikaz.CreateConnectionWithTrans())
                {
                    try
                    {
                        r.ErrorCode = -6;
                        if (lb_update)
                        {
                            KSProgram.Sluzebky.SlcHlavicka.Update(tabulka, ev_cislo, ev_cislo, null, c.Cislo, os_cislo, c.Typ,
                                c.PocMisto, c.DatOd.Date, c.DatOd.ToString("HH:mm"), c.KonMisto, c.DatDo.Date, c.DatDo.ToString("HH:mm"),
                                c.Cil, c.CilZeme, c.Ucel,
                                spolucestujici, spolu[0], spolu[1], spolu[2], spolu[3], spolu[4], spolu[5], spolu[6], spolu[7], spolu[8], spolu[9],
                                c.DruhDopr, c.Spz, /*"", "",*/ /*ldc_zakl_nahr,*/
                                ls_phm, /*ldc_cena_phm,*/ ldc_prum_spotreba, ldc_spotreba_mesto, ldc_spotreba_mimo, ls_pov_ruc, c.Poznamka, null, null, 
                                c.DruhNakl, c.PredpCelkNakl, c.PCNKodMeny, null);

                            if (c.Zalohy != null) KSProgram.Sluzebky.SlcZalohy.DeleteAll(tabulka, ev_cislo, /*i_vyplacene*/ false);
                            if (c.Prubeh != null) KSProgram.Sluzebky.SlcPrubeh.DeleteAll(ev_cislo);

                        }
                        else
                        {
                            KSProgram.Sluzebky.SlcHlavicka.Insert(tabulka, out ev_cislo, null, os_cislo, zdroj, c.Typ,
                                c.PocMisto, c.DatOd.Date, c.DatOd.ToString("HH:mm"), c.KonMisto, c.DatDo.Date, c.DatDo.ToString("HH:mm"),
                                c.Cil, c.CilZeme, c.Ucel,
                                spolucestujici, spolu[0], spolu[1], spolu[2], spolu[3], spolu[4], spolu[5], spolu[6], spolu[7], spolu[8], spolu[9],
                                c.DruhDopr, c.Spz, /*"", "",*/ /*ldc_zakl_nahr,*/
                                ls_phm, /*ldc_cena_phm,*/ ldc_prum_spotreba, ldc_spotreba_mesto, ldc_spotreba_mimo, ls_pov_ruc,
                                c.DatumSouhlasu, c.PovolilDne, ldc_povolil, c.Poznamka, null, null, c.DruhNakl, c.PredpCelkNakl, c.PCNKodMeny, null);
                            DateTime ee = new DateTime(2016, 7, 20, 8, 30, 0) ;
                            if (!String.IsNullOrEmpty(c.Cislo))
                            {   //zmena cisla SC
                                SqlPrikaz.ExecuteNonQuery("update " + tabulka + " set cislo='" + c.Cislo + "' where ev_cislo=" + ev_cislo);
                                //PR 10.7.2015 zapis cisla do dbarchu, aby se u smazane SC dalo dohledat ev_cislo dle externiho cisla
                                /*KM 7.10.2015 string podm = "ev_cislo=" + ev_cislo + " and os_cislo=" + os_cislo;
                                DBArch.Insert(tabulka, "cislo", ev_cislo.ToString(), c.Cislo, podm, "SCesty2.CestaVloz");*/
                            }
                            else
                                c.Cislo = ev_cislo.ToString();
                        }
                        r.ErrorCode = -7;
                        if (c.Vytvoril != null && (!String.IsNullOrEmpty(c.Vytvoril.Login) || c.Vytvoril.OsCislo != null))
                        {
                            string ls_vytvoril;
                            if (!String.IsNullOrEmpty(c.Vytvoril.Login))
                                ls_vytvoril = c.Vytvoril.Login;
                            else
                            {
                                if (c.Vytvoril.OsCislo == null || c.Vytvoril.OsCislo == 0)
                                    ls_vytvoril = "system";
                                else
                                    ls_vytvoril = SpravaUziv.UzivatelPodleOsobnihoCisla(c.Vytvoril.OsCislo.Value);
                            }
                            SqlPrikaz.ExecuteNonQuery("update " + tabulka + " set vytvoril=" + SqlPrikaz.ValueToString(ls_vytvoril)
                                    + ", vytvor_dne=" + SqlPrikaz.ValueToString(c.VytvorDne)
                                    + " where ev_cislo=" + SqlPrikaz.ValueToString(ev_cislo));
                        }
                        r.ErrorCode = -8;
                        //PR 17.1.2018 zálohy vráceny před workflow, protože se zapisovaly do workfl_slc_zalohy a až po proběhnutí workflow akcí zal_schval
                        if (c.Zalohy != null && c.Zalohy.Count > 0)
                        {                                                       
                            if (lb_update) SlcZalohy.DeleteAll(ev_cislo, /*i_vyplacene*/ false); //PR 9.3.2017 mažu jen nevyplacené zálohy

                            string tab_zal;
                            if (tabulka == "workfl_sluz_cesty")
                                tab_zal = "workfl_slc_zalohy";
                            else
                                tab_zal = "slc_zalohy";

                            foreach (SCZaloha z in c.Zalohy)
                            {
                                int zalId = 0;
                                DateTime? vypl_dne = null;
                                string vypl_jak = null;
                                zalId = SlcZalohy.Find(tab_zal, ev_cislo, z.Castka, z.KodMeny, z.Druh, out vypl_dne, out vypl_jak); //PR 28.3.2017 ev_cislo
                                if (z.VyplJak == "") z.VyplJak = null;
                                if (vypl_jak == "") vypl_jak = null;
                                if (zalId == 0)
                                    SlcZalohy.Insert(tab_zal, ev_cislo, out zalId, z.Druh, z.Castka, z.KodMeny,
                                        z.PozadalDne, OsobaNaOsCislo(z.Pozadal), z.SchvalilDne, OsobaNaOsCislo(z.Schvalil), 
                                        z.VyplatilDne, OsobaNaOsCislo(z.Vyplatil), z.VyplJak, z.VyplDoklad, null);
                                else if (zalId > 0 && (vypl_dne == null || (z.VyplatilDne != null && vypl_dne == z.VyplatilDne)) && (vypl_jak == z.VyplJak || (vypl_jak == null && z.VyplJak != null)))
                                    //PR 9.3.2017 jen pokud není vyplacena nebo se nemění datum vyplacení,
                                    //a zároveň se nemění způsob vyplacení z jednoho typu na jiný, tak ji updatnu (jinak ji ponechám bezezměny)
                                    SlcZalohy.Update(tab_zal, zalId, zalId, ev_cislo, z.Druh, z.Castka, z.KodMeny,
                                        z.PozadalDne, OsobaNaOsCislo(z.Pozadal), z.SchvalilDne, OsobaNaOsCislo(z.Schvalil), 
                                        z.VyplatilDne, OsobaNaOsCislo(z.Vyplatil), z.VyplJak, z.VyplDoklad, null);
                            }
                        }
                        r.ErrorCode = -9;
                        if (pouzitWorkFlow)
                        {
                            workfl_stavy lo_validniStavy = new workfl_stavy(5);
                            int id_poz = 0;
                            int? id_poztmp = Pozadaveksluzc.EvCisloToIdPozadavku(ev_cislo);
                            if (id_poztmp != null && id_poztmp > 0) id_poz = id_poztmp.Value;
                            if (id_poz == 0)
                                PozadavekInsert(ref id_poz, os_cislo, c.DatOd, c.DatDo, c.Cil, c.Ucel, c.DruhDopr, tabulka, ev_cislo, null);

                            //Pozadaveksluzc p = new Pozadaveksluzc();
                            //p.Retrieve(idPoz);
                            int posledni = c.WorkFlow.Count - 1;
                            for (int poradi = 0; poradi < c.WorkFlow.Count; poradi++)
                            {

                                SCWorkflowProc p = c.WorkFlow[poradi];
                                decimal ldc_resitel = OsobaNaOsCislo(p.Resitel);

                                Str_workfl_proc str = new Str_workfl_proc();
                                str.id_poz = id_poz;
                                str.poradi = poradi + 1;

                                foreach (Str_workfl_stavy st in lo_validniStavy.Stavy)
                                {
                                    if (st.stav.Equals(p.Stav))
                                    {
                                        str.faze = st.faze;
                                        str.stav = st.stav;
                                        str.substav = "";
                                        str.sys_stav = st.sys_stav;
                                        str.faze_akt = st.next_faze; //PR 3.10.2016 
                                        if (poradi == posledni) //PR 7.5.2015 
                                        {
                                            //str.adresat_neupr = st.adresat;//PR 3.10.2016 
                                            //str.adresat = WorkflAdresat.UpravAdresataProWorkflProc(id_poz, st.adresat);//PR 11.2.2015 
                                            //str.oznam = st.oznam;//PR 3.10.2016 
                                            //PR 3.11.2017 Adresát se vezme dle nastavení workflow
                                        }
                                        else//PR 7.5.2015 
                                        {
                                            str.adresat = "[oc=" + OsobaNaOsCislo(c.WorkFlow[poradi + 1].Resitel) + "]";
                                            str.adresat_neupr = str.adresat;//PR 3.10.2016
                                            str.oznam = "";//PR 3.10.2016
                                        }
                                        break;
                                    }
                                }

                                str.resitel = ldc_resitel;
                                str.datum = p.Datum;
                                //str.adresat = (poradi < posledni) ? "[!vyd]" : "[oc=" + os_cislo + "]";
                                str.duvod = p.Duvod;

                                //pozn: tj. MAIL se neodešle nikdy, porovnání Today vracející čas 00:00 se neshodne s časem v str.datum
                                if (WfProcInsert(str, /*sendmail*/poradi == posledni && str.datum == DateTime.Today)) 
                                {
                                    //if (p.Stav == "novy" && os_cislo == ldc_resitel)
                                    //{
                                    //    SqlPrikaz.ExecuteNonQuery("update " + tabulka + " set souhl_dne=" + SqlPrikaz.ValueToString(p.Datum)
                                    //    + " where ev_cislo=" + SqlPrikaz.ValueToString(ev_cislo));
                                    //}
                                    //if (p.Stav == "povolena")
                                    //{
                                    //    SqlPrikaz.ExecuteNonQuery("update " + tabulka + " set povolil_osc=" + SqlPrikaz.ValueToString(ldc_resitel)
                                    //    + ", povolil_dne=" + SqlPrikaz.ValueToString(p.Datum)
                                    //    + " where ev_cislo=" + SqlPrikaz.ValueToString(ev_cislo));

                                    //    if (tabulka == "workfl_sluz_cesty")
                                    //    {
                                    //        tabulka = "sluz_cesty";
                                    //        SqlPrikaz.ExecuteNonQuery("insert into sluz_cesty select * from workfl_sluz_cesty where ev_cislo=" + SqlPrikaz.ValueToString(ev_cislo));
                                    //        SqlPrikaz.ExecuteNonQuery("update workfl_body set tabulka='sluz_cesty' where archiv is null and id_poz=" + SqlPrikaz.ValueToString(id_poz));
                                    //    }
                                    //}
                                    ////odeslání mailu pro poslední krok (vypnout si to mohou v nastavení workflow)
                                    //if (poradi == posledni && lb_update == false)
                                    //{
                                    //    WorkflowProces.CreateAndSendMail(id_poz);
                                    //}
                                }
                            }
                        }//WF

                        //PR 17.1.2018 posunuto před workflow 
                        //if (c.Zalohy != null && c.Zalohy.Count > 0)
                        //{
                        //    r.ErrorCode = -9;
                        //    //if (lb_update) SlcZalohy.DeleteAll(ev_cislo);
                        //    //foreach (SCZaloha z in c.Zalohy)
                        //    //{
                        //    //    int zalId;
                        //    //    SlcZalohy.Insert(tabulka, ev_cislo, out zalId, z.Druh, z.Castka, z.KodMeny,
                        //    //        z.PozadalDne, OsobaNaOsCislo(z.Pozadal), z.SchvalilDne, OsobaNaOsCislo(z.Schvalil), z.VyplatilDne, OsobaNaOsCislo(z.Vyplatil), z.VyplJak, z.VyplDoklad);
                        //    //}
                        //    //PR 9.3.2017
                        //    if (lb_update) SlcZalohy.DeleteAll(ev_cislo, /*i_vyplacene*/ false); //PR 9.3.2017 mažu jen nevyplacené zálohy

                        //    string tab_zal;
                        //    if (tabulka == "workfl_sluz_cesty")
                        //        tab_zal = "workfl_slc_zalohy";
                        //    else
                        //        tab_zal = "slc_zalohy";

                        //    foreach (SCZaloha z in c.Zalohy)
                        //    {
                        //        int zalId = 0;
                        //        DateTime? vypl_dne = null;
                        //        string vypl_jak = null;
                        //        zalId = SlcZalohy.Find(tab_zal, ev_cislo, z.Castka, z.KodMeny, z.Druh, out vypl_dne, out vypl_jak); //PR 28.3.2017 ev_cislo
                        //        if (z.VyplJak == "") z.VyplJak = null;
                        //        if (vypl_jak == "") vypl_jak = null;
                        //        if (zalId == 0)
                        //            SlcZalohy.Insert(tab_zal, ev_cislo, out zalId, z.Druh, z.Castka, z.KodMeny,
                        //                z.PozadalDne, OsobaNaOsCislo(z.Pozadal), z.SchvalilDne, OsobaNaOsCislo(z.Schvalil), z.VyplatilDne, OsobaNaOsCislo(z.Vyplatil), z.VyplJak, z.VyplDoklad);
                        //        else if (zalId > 0 && (vypl_dne == null || (z.VyplatilDne != null && vypl_dne == z.VyplatilDne)) && (vypl_jak == z.VyplJak || (vypl_jak==null && z.VyplJak!=null)))
                        //            //PR 9.3.2017 jen pokud není vyplacena nebo se nemění datum vyplacení,
                        //            //a zároveň se nemění způsob vyplacení z jednoho typu na jiný, tak ji updatnu (jinak ji ponechám bezezměny)
                        //            SlcZalohy.Update(tab_zal, zalId, zalId, ev_cislo, z.Druh, z.Castka, z.KodMeny,
                        //                z.PozadalDne, OsobaNaOsCislo(z.Pozadal), z.SchvalilDne, OsobaNaOsCislo(z.Schvalil), z.VyplatilDne, OsobaNaOsCislo(z.Vyplatil), z.VyplJak, z.VyplDoklad);
                        //    }
                        //}

                        if (c.Prubeh != null && c.Prubeh.Count > 0)
                        {
                            r.ErrorCode = -10;
                            if (lb_update) SlcPrubeh.DeleteAll(ev_cislo);
                            int ll_r = 0;
                            foreach (SCPrubeh2 p in c.Prubeh)
                            {
                                ll_r++; //PR 17.01.2022 novy sloupec poradi
                                SlcPrubeh.Insert(ev_cislo, ll_r, ll_r, p.Udalost, p.Datum, p.Datum.ToString("HH:mm"), p.Misto, p.ZemeOd, p.ZemeDo, p.DruhDopr, p.KmMesto, p.KmMimo, p.KmKomb, p.KmCelkem);
                            }
                        }

                        if (c.Vydaje != null && c.Vydaje.Count>0)
                        {
                            r.ErrorCode = -11;
                            if (lb_update) SlcNahrady.DeleteAll(ev_cislo);
                            foreach (SCVydaj2 v in c.Vydaje)
                            {
                                int vydId;
                                SlcNahrady.Insert(out vydId, ev_cislo, v.Datum, v.Druh, v.ZpusPlat, v.Pozn, v.Castka, v.KodMeny, v.KodDPH);
                            }
                        }
                        SqlPrikaz.Commit();
                    }
                    catch (Exception ex)
                    {
                        SqlPrikaz.Rollback();
                        //r.ErrorCode = -1;
                        r.ErrorString = ex.Message;
                        if (null != ex.InnerException) r.ErrorString += ";\r\nInnerException=" + ex.InnerException.Message;
                        if (null != ex.Source) r.ErrorString += ";\r\nSource=" + ex.Source;
                        if (null != ex.StackTrace) r.ErrorString += ";\r\nStackTrace=" + ex.StackTrace;
                        return r;
                    }
                }
            }
            catch (Exception ex)
            {
                //r.ErrorCode = -1;
                r.ErrorString = ex.Message;
                if (null != ex.InnerException) r.ErrorString += ";\r\nInnerException=" + ex.InnerException.Message;
                if (null != ex.Source) r.ErrorString += ";\r\nSource=" + ex.Source;
                if (null != ex.StackTrace) r.ErrorString += ";\r\nStackTrace=" + ex.StackTrace;
                return r;
            }
            r.ErrorCode = 0;
            r.ErrorString = "OK";
            if (!lb_CisloNaVstupu) r.ErrorString += " (Cislo=" + c.Cislo + ")";
            return r;
        }

        private SCResult CestaValidaceHodnot(ref SCCesta2 c)
        {
            SCResult r = new SCResult();

            c.Cislo = Trunc(c.Cislo, 25);
            if (null == c.Zamestnanec)
            {
                r.ErrorCode = -1;
                r.ErrorString = "Není zadán zaměstnanec (SCCesta2.Zamestnanec).";
                return r;
            }
            Decimal os_cislo = OsobaNaOsCislo(c.Zamestnanec);
            if (!Pracovnici.ExistOsCislo(os_cislo))
            {
                r.ErrorCode = -1;
                r.ErrorString = "Zaměstnanec s os. číslem " + os_cislo + " neexistuje.";
                return r;
            }
            Globals.CreateRadPrava(os_cislo/*, DateTime.Now*/); //PR 7.5.2015 aby se později nevytvářely pro os_cislo=0, což by hodilo vyjímku
            
            if (c.DatOd.Year < 2000 || c.DatOd.Year > 2100)
            {
                r.ErrorCode = -1;
                r.ErrorString = "Není zadán datum počátku SC (SCCesta2.DatOd).";
                return r;
            }
            if (c.DatDo.Year < 2000 || c.DatDo.Year > 2100)
            {
                r.ErrorCode = -1;
                r.ErrorString = "Není zadán datum konce SC (SCCesta2.DatDo).";
                return r;
            }
            if (String.IsNullOrEmpty(c.Typ))
            {
                r.ErrorCode = -1;
                r.ErrorString = "Není zadán typ SC (SCCesta2.Typ={t,z}).";
                return r;
            }
            c.Typ = c.Typ.ToLower();
            if (c.Typ != "t" && c.Typ != "z")
            {
                r.ErrorCode = -1;
                r.ErrorString = "Neplatný typ SC (SCCesta2.Typ={t,z}).";
                return r;
            }
            if (String.IsNullOrEmpty(c.Cil))
            {
                r.ErrorCode = -1;
                r.ErrorString = "Není zadán cíl (SCCesta2.Cil).";
                return r;
            }
            c.Cil = Trunc(c.Cil, 80);
            if (String.IsNullOrEmpty(c.CilZeme) == false)
            {
                c.CilZeme = Trunc(c.CilZeme, 45);
                if (c.Typ == "t")
                    c.CilZeme = SlcCisZemi.TUZEMSKO();
                else
                {
                    if (!SlcCisZemi.Existuje(c.CilZeme))
                    {
                        r.ErrorCode = -1;
                        r.ErrorString = "Neplatná země (SCCesta2.CilZeme), země musí být dle vyhlášky - základní sazby zahraničního stravného.";
                        return r;
                    }
                }
            }
            if (String.IsNullOrEmpty(c.Ucel))
            {
                r.ErrorCode = -1;
                r.ErrorString = "Není zadán účel (SCCesta2.Ucel).";
                return r;
            }
            c.Ucel = Trunc(c.Ucel, 80);
            c.PocMisto = Trunc(c.PocMisto, 40);
            c.KonMisto = Trunc(c.KonMisto, 40);

            if (String.IsNullOrEmpty(c.DruhDopr))
            {
                r.ErrorCode = -1;
                r.ErrorString = "Není zadán Druh dopr. prostředku (SCCesta2.DruhDopr).";
                return r;
            }
            else //více druhy oddělenými čárkou
            {
                string[] ls_d = c.DruhDopr.Split(',');
                foreach (string d in ls_d)
                {
                    if (String.IsNullOrEmpty(SlcCisDruhDopr.Nazev(d)))
                    {
                        r.ErrorCode = -1;
                        r.ErrorString = "Neplatný druh dopr. prostředku " + d + " (SCCesta2.DruhDopr).";
                        return r;
                    }
                }
            }
            c.Poznamka = Trunc(c.Poznamka, 120);
            if (!string.IsNullOrEmpty(c.DruhNakl))
            {
                if (string.IsNullOrEmpty(KSProgram.Ciselniky.CisUniv.Nazev(105, c.DruhNakl)))
                {
                    r.ErrorCode = -1;
                    r.ErrorString = "Neplatný druh nákladu " + c.DruhNakl + " (SCCesta2.DruhNakl).";
                    return r;
                }
            }

            //validace zaloh
            if (c.Zalohy != null) //PR 12.2.2018 
                foreach (SCZaloha z in c.Zalohy)
                {
                    if (!String.IsNullOrEmpty(z.Druh))
                    {
                        if (String.IsNullOrEmpty(SlcCisCn.Nazev(z.Druh)))
                        {
                            r.ErrorCode = -1;
                            r.ErrorString = "Neplatný druh zálohy " + z.Druh + " (SCZaloha.Druh).";
                            return r;
                        }
                    }
                    if (!CisKodMeny.Existuje(z.KodMeny))
                    {
                        r.ErrorCode = -1;
                        r.ErrorString = "Neplatný kód měny zálohy " + z.KodMeny + " (SCZaloha.KodMeny).";
                        return r;
                    }
                    if (!String.IsNullOrEmpty(z.VyplJak))//PR 31.7.2014 
                    {
                        if (String.IsNullOrEmpty(SlcCisZpusPlat.Nazev(z.VyplJak)))
                        {
                            r.ErrorCode = -1;
                            r.ErrorString = "Neplatný způsob vyplacení zálohy " + z.VyplJak + " (SCZaloha.VyplJak).";
                            return r;
                        }
                    }
                    z.VyplDoklad = Trunc(z.VyplDoklad, 15);
                }

            //validace prubehu
            int ll_row = 0;
            string ls_udalost_pred = "", ls_zeme_pred = "", ls_dopr_pred = "";
            if (c.Prubeh != null) //PR 12.2.2018 
                foreach (SCPrubeh2 p in c.Prubeh)
                {
                    ll_row++;
                    //jednoduche kontroly uvnitr radku
                    if (String.IsNullOrEmpty(p.Udalost) || "OPHVKRU".Contains(p.Udalost) == false)
                    {
                        r.ErrorCode = -1;
                        if (String.IsNullOrEmpty(p.Udalost))
                            r.ErrorString = "Průběh ř. " + ll_row + " - Není zadána položka událost";
                        else
                            r.ErrorString = "Průběh ř. " + ll_row + " - Nesprávná hodnota v poli událost";
                        r.ErrorString += " (SCPrubeh2.Udalost={O, P, H, V, K, R, U}).";
                        return r;
                    }

                    if (p.Datum < c.DatOd || p.Datum > c.DatDo)
                    {
                        r.ErrorCode = -1;
                        r.ErrorString = "Průběh ř. " + ll_row + " - Datum v části průběh není v intervalu konání SC (SCPrubeh2.Datum).";
                        return r;
                    }

                    if (p.Udalost == "O" || p.Udalost == "H" || p.Udalost == "P")
                    {
                        if (String.IsNullOrEmpty(p.Misto))
                        {
                            r.ErrorCode = -1;
                            r.ErrorString = "Průběh ř. " + ll_row + " - Není zadáno místo (SCPrubeh2.Misto).";
                            return r;
                        }
                        p.Misto = Trunc(p.Misto, 40);

                        if (!String.IsNullOrEmpty(p.ZemeOd))
                        {
                            if (p.ZemeOd != SlcCisZemi.TUZEMSKO() && !SlcCisZemi.Existuje(p.ZemeOd))
                            {
                                r.ErrorCode = -1;
                                r.ErrorString = "Průběh ř. " + ll_row + " - Neplatný název země " + p.ZemeOd + " (SCPrubeh2.ZemeOd).";
                                return r;
                            }
                            p.ZemeOd = Trunc(p.ZemeOd, 45);
                        }
                        if (!String.IsNullOrEmpty(p.ZemeDo))
                        {
                            if (p.ZemeDo != SlcCisZemi.TUZEMSKO() && !SlcCisZemi.Existuje(p.ZemeDo))
                            {
                                r.ErrorCode = -1;
                                r.ErrorString = "Průběh ř. " + ll_row + " - Neplatný název země " + p.ZemeDo + " (SCPrubeh2.ZemeDo).";
                                return r;
                            }
                            p.ZemeDo = Trunc(p.ZemeDo, 45);
                        }
                        if (p.Udalost == "H")
                        {
                            if (!String.IsNullOrEmpty(p.ZemeOd) && !String.IsNullOrEmpty(p.ZemeDo) && p.ZemeOd == p.ZemeDo)
                            {
                                r.ErrorCode = -1;
                                r.ErrorString = "Průběh ř. " + ll_row + " - U přechodu hranic se „Země z“ nemůže rovnat „Zemi do“ (SCPrubeh2.ZemeDo)";
                                return r;
                            }
                        }
                        if (String.IsNullOrEmpty(p.DruhDopr))
                        {
                            r.ErrorCode = -1;
                            r.ErrorString = "Průběh ř. " + ll_row + " - Není zadán Druh dopr. prostředku (SCPrubeh2.DruhDopr).";
                            return r;
                        }
                        else if (String.IsNullOrEmpty(SlcCisDruhDopr.Nazev(p.DruhDopr)))
                        {
                            r.ErrorCode = -1;
                            r.ErrorString = "Průběh ř. " + ll_row + " - Neplatný druh dopr. prostředku " + p.DruhDopr + " (SCPrubeh2.DruhDopr).";
                            return r;
                        }
                    }

                    //kontroly oproti predchozimu radku
                    if (ls_udalost_pred.Length > 0)
                    {
                        if (ls_udalost_pred == "O" && (p.Udalost != "H" && p.Udalost != "P"))
                        {
                            r.ErrorCode = -1;
                            r.ErrorString = "Průběh ř. " + ll_row + " - Po události Odjezd může následovat pouze Příjezd nebo Přechod hranic (SCPrubeh2.Udalost)";
                            return r;
                        }

                        if (p.Udalost == "H" && (ls_udalost_pred != "O" && ls_udalost_pred != "H"))
                        {
                            r.ErrorCode = -1;
                            r.ErrorString = "Průběh ř. " + ll_row + " - Přechod hranic může následovat pouze za odjezdem nebo přechodem hranic (SCPrubeh2.Udalost)";
                            return r;
                        }

                        if (p.Udalost == ls_udalost_pred && p.Udalost != "H")
                        {
                            r.ErrorCode = -1;
                            r.ErrorString = "Průběh ř. " + ll_row + " - Událost typu " + p.Udalost + " nemůže být vícekrát zasebou (SCPrubeh2.Udalost)";
                            return r;
                        }

                        if (p.Udalost == "O" || p.Udalost == "H" || p.Udalost == "P")
                        {
                            if (p.ZemeOd != ls_zeme_pred)
                            {
                                r.ErrorCode = -1;
                                r.ErrorString = "Průběh ř. " + ll_row + " - Země " + p.ZemeOd + " se neshoduje se zemí v předešlém záznamu majícím událost O,H,P (SCPrubeh2.ZemeOd).";
                                return r;
                            }
                        }
                    }

                    //====== fix values ======
                    //DOPRAVA - pokud je udalost P nebo H, tak doprava musi byt stejna jako u radku s udalosti O
                    if ((p.Udalost == "H" || p.Udalost == "P") && p.DruhDopr != ls_dopr_pred)
                        p.DruhDopr = ls_dopr_pred;

                    //ZEME_DO - pokud zeme_do neni visible (visible je u hranice nebo odlet+letadlo), tak musi mit stejnou hodnotu jako zeme_od
                    if (!(p.Udalost == "H" || (p.Udalost == "O" && p.DruhDopr == "L")))
                    {
                        p.ZemeDo = p.ZemeOd;
                    }

                    ls_udalost_pred = p.Udalost;
                    if (p.Udalost == "O" || p.Udalost == "H" || p.Udalost == "P")
                    {
                        ls_zeme_pred = p.ZemeDo;
                        ls_dopr_pred = p.DruhDopr;
                    }
                }

            //validace workflow //PR 7.8.2014 
            bool pouzitWorkFlow = (c.WorkFlow != null && c.WorkFlow.Count > 0);
            if (pouzitWorkFlow)
            {
                workfl_stavy lo_validniStavy = new workfl_stavy(5);
                foreach (SCWorkflowProc p in c.WorkFlow)
                {
                    if (String.IsNullOrEmpty(p.Stav))
                    {
                        r.ErrorCode = -1;
                        r.ErrorString = "Není zadán Stav (SCWorkflowProc.Stav).";
                        return r;
                    }
                    //PR 11.2.2015 zrušena dočasná vyjímka
                    //else if (p.Stav.Equals("povoleno"))
                    //    p.Stav = "povolena";
                    bool lb_valid = false;
                    foreach (Str_workfl_stavy s in lo_validniStavy.Stavy)
                    {
                        if (s.stav.Equals(p.Stav))
                        {
                            lb_valid = true;
                            break;
                        }
                    }
                    if (!lb_valid)
                    {
                        r.ErrorCode = -1;
                        r.ErrorString = "Neplatný druh stavu " + p.Stav + " (SCWorkflowProc.Stav).";
                        return r;
                    }

                    /*if (!(p.Stav.Equals("novy") || p.Stav.Equals("povolena")))
                    {
                        r.ErrorCode = -1;
                        r.ErrorString = "Neplatný druh stavu " + p.Stav + ", podporované jsou pouze novy a povolena (SCWorkflowProc.Stav).";
                        return r;
                    }*/
                    if (p.Resitel == null)
                    {
                        r.ErrorCode = -1;
                        r.ErrorString = "Není zadán Resitel (SCWorkflowProc.Resitel), pro ne-uživatele portálu nastav Resitel.OsCislo=0.";
                        return r;
                    }
                    else if (p.Resitel.OsCislo == null && String.IsNullOrEmpty(p.Resitel.Login))
                    {
                        r.ErrorCode = -1;
                        r.ErrorString = "Není zadán Resitel.OsCislo ani Resitel.Login (SCWorkflowProc.Resitel), pro ne-uživatele portálu nastav Resitel.OsCislo=0.";
                        return r;
                    }
                }
            }

            return r;
        }

        /// <summary>
        /// Smazání služební cesty.
        /// </summary>
        /// <param name="cislo"></param>
        /// <returns></returns>
        [WebMethod(EnableSession = true, Description = "Smazání služební cesty.")]
        [SoapHeader("AuthHeader")]
        public SCResult CestaSmaz2(string Cislo, decimal OsCislo)
        {
            SCResult r = new SCResult();
            r.ErrorCode = -99;
            r.ErrorString = "Chyba";

            try
            {
                //PRIPOJENI K DB
                if (null == Globals.Db)
                {
                    Globals.Db = DatabasePodleParam.Create(null);
                    KSTrace.WriteLine("Aktivní DB", Globals.Db.ConnectionStringWithoutCredentials);
                }

                //AUTENTIZACE
                //if (!this.User.Identity.IsAuthenticated)
                //string at = this.User.Identity.AuthenticationType == "" ? "anonymous" : this.User.Identity.AuthenticationType;
                string authMessage = Authentication(AuthHeader);
                if (authMessage != null)
                {
                    r.ErrorCode = -401;
                    r.ErrorString = authMessage;
                    return r;
                }

                //VALIDACE HODNOT
                //PR 11.2.2015 uvolněno pro VUANCH
                //PR 05.12.2018 ACO
                if (!(Zakaznik.IsCurrent("VARROC") || Zakaznik.IsCurrent("BLANKYT") || Zakaznik.IsCurrent("VUANCH") || Zakaznik.IsCurrent("ACO")))
                {
                    r.ErrorCode = -3;
                    r.ErrorString = "Metoda není uvolněna pro vaší organizaci.";
                    return r;
                }

                if (String.IsNullOrEmpty(Cislo))
                {
                    r.ErrorCode = -11;
                    r.ErrorString = "Není zadáno číslo SC.";
                    return r;
                }

                if (OsCislo <= 0)
                {
                    r.ErrorCode = -12;
                    r.ErrorString = "Není zadáno osobní číslo.";
                    return r;
                }

                int ev_cislo = 0;
                ev_cislo = SqlPrikaz.ExecuteScalarInt32("select ev_cislo from sluz_cesty where cislo='" + Cislo + "' and os_cislo=" + OsCislo, 0);
                if (ev_cislo == 0)
                    ev_cislo = SqlPrikaz.ExecuteScalarInt32("select ev_cislo from workfl_sluz_cesty where cislo='" + Cislo + "' and os_cislo=" + OsCislo, 0);


                if (ev_cislo == 0)
                {
                    r.ErrorCode = -13;
                    r.ErrorString = "SC nenalezena (cislo='" + Cislo + "' and os_cislo=" + OsCislo + ").";
                    return r;
                }

                if (SlcFunkce.JeVyuctovana(ev_cislo)) //PR 16.10.2014 
                {
                    r.ErrorCode = -14;
                    r.ErrorString = "Nelze smazat: cestovní příkaz již byl vyúčtován!";
                    return r;
                }
                else
                {
                    int? id_poz = Pozadaveksluzc.EvCisloToIdPozadavku(ev_cislo);
                    if (id_poz.HasValue && id_poz.Value > 0)
                    {
                        Pozadaveksluzc poz = new Pozadaveksluzc();
                        poz.DeleteBase(id_poz.Value);
                    }
                    else
                        SlcFunkce.DeleteAllData(ev_cislo);
                }

            }
            catch (Exception ex)
            {
                //r.ErrorCode = -1;
                r.ErrorString = ex.Message;
                if (null != ex.InnerException) r.ErrorString += ";\r\nInnerException=" + ex.InnerException.Message;
                if (null != ex.Source) r.ErrorString += ";\r\nSource=" + ex.Source;
                if (null != ex.StackTrace) r.ErrorString += ";\r\nStackTrace=" + ex.StackTrace;
                return r;
            }
            r.ErrorCode = 0;
            r.ErrorString = "OK";
            return r;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="s"></param>
        /// <param name="len"></param>
        /// <returns></returns>
        private string Trunc(string s, int len)
        {
            if (String.IsNullOrEmpty(s)) return s;
            if (s.Length > len) s = s.Substring(0, len);
            return s;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="o"></param>
        /// <returns></returns>
        private decimal OsobaNaOsCislo(SCOsoba o)
        {
            if (null == o) return 0;

            decimal osCislo;
            if (o.OsCislo != null /*&& o.OsCislo != 0*/) //PR 16.12.2014
            {
                osCislo = o.OsCislo.Value;
            }
            else
            {
                List<StrOsCisloPodnik> str_os_cisla = new List<StrOsCisloPodnik>();
                osCislo = SpravaUziv.OsCisloUzivatele(o.Login, ref str_os_cisla);
            }
            return osCislo;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="id_poz"></param>
        /// <param name="os_cislo"></param>
        /// <param name="datum_od"></param>
        /// <param name="datum_do"></param>
        /// <param name="cil"></param>
        /// <param name="ucel"></param>
        /// <param name="druh_dopr"></param>
        /// <param name="tabulka"></param>
        /// <param name="ev_cislo"></param>
        /// <param name="vyridit_do"></param>
        public static void PozadavekInsert(ref int id_poz, decimal os_cislo, DateTime? datum_od, DateTime? datum_do, string cil, string ucel, string druh_dopr, string tabulka, int ev_cislo, DateTime? vyridit_do)
        {
            cil = InputChecker.Text(cil, "cil");
            ucel = InputChecker.Text(ucel, "ucel");
            druh_dopr = InputChecker.Text(druh_dopr, "druh_dopr");
            tabulka = InputChecker.Text(tabulka, "tabulka");

            if (datum_do == new DateTime()) datum_do = null;
            if (datum_od == new DateTime()) datum_od = null;
            if (vyridit_do == new DateTime()) vyridit_do = null;


            if (vyridit_do == null)
                vyridit_do = datum_od;


            string ls_stav = "";
            //PR 6.8.2014 přidal jsem os_cislo
            id_poz = Pozadaveksluzc.Workfl_hl_insert("sluzc", os_cislo, "workfl_body", ref ls_stav);

            DateTime obdobi = new DateTime(((DateTime)datum_od).Year, ((DateTime)datum_od).Month, 1);
            string sql = "insert into workfl_body (id, id_poz, verze, datum_od, datum_do, " +
                                    " popis, duvod, popis2, tabulka, id_vazba, vyridit_do, obdobi, datum, zapsal)" +
                " values({0}," + id_poz + ",1," + SqlPrikaz.ValueToString(datum_od) + "," +
                        SqlPrikaz.ValueToString(datum_do) + "," +
                        SqlPrikaz.ValueToString(cil) + "," +
                        SqlPrikaz.ValueToString(ucel) + "," +
                        SqlPrikaz.ValueToString(druh_dopr) + "," +
                        SqlPrikaz.ValueToString(tabulka) + "," +
                        SqlPrikaz.ValueToString(ev_cislo) + "," +
                        SqlPrikaz.ValueToString(vyridit_do) + "," +
                        SqlPrikaz.ValueToString(obdobi) + "," +
                        SqlPrikaz.ValueToString(DateTime.Today.Date) + "," +
                        SqlPrikaz.ValueToString(0) + ")";
            //SqlPrikaz.ExecuteNonQuery(sql);
            SqlPrikaz.InsertWithSerial(sql, "workfl_body", "id");
        }

        /// <summary>
        /// Vložení záznamu do workfl_proc
        /// </summary>
        /// <param name="str"></param>
        /// <param name="sendmail"></param>
        /// <returns></returns>
        private static bool WfProcInsert(Str_workfl_proc str, bool sendmail)
        {
            int ll_exist = SqlPrikaz.ExecuteScalarInt32("select count(*) from workfl_proc " +
                                    " where id_poz=" + SqlPrikaz.ValueToString(str.id_poz) + " and stav=" + SqlPrikaz.ValueToString(str.stav), 0);

            string s_cas = F_DateTime.Cas(str.datum);

            if (ll_exist == 0)
            {
                //SqlPrikaz.ExecuteNonQuery("update workfl_proc set archiv='a' " +
                //                        " where id_poz=" + SqlPrikaz.ValueToString(str.id_poz) + " and stav<>'diskuse'");//PR 6.8.2014 presunuto
                //string sql = "insert into workfl_proc (id, id_poz, poradi, stav, sys_stav, substav, faze, adresat, duvod, resitel, datum, cas, zapsal)" +
                //        " values (" +
                //                "{0}," +
                //                SqlPrikaz.ValueToString(str.id_poz) + "," +
                //                SqlPrikaz.ValueToString(str.poradi) + "," +
                //                SqlPrikaz.ValueToString(str.stav) + "," +
                //                SqlPrikaz.ValueToString(str.sys_stav) + "," +
                //                SqlPrikaz.ValueToString(str.substav) + "," +
                //                SqlPrikaz.ValueToString(str.faze) + "," +
                //                SqlPrikaz.ValueToString(str.adresat) + "," +
                //                SqlPrikaz.ValueToString(str.duvod) + "," +
                //                SqlPrikaz.ValueToString(str.resitel) + "," +
                //                SqlPrikaz.ValueToString(str.datum) + "," +
                //                SqlPrikaz.ValueToString(s_cas) + "," +
                //                SqlPrikaz.ValueToString(str.zapsal) +
                //            ")";
                //int id = SqlPrikaz.InsertWithSerial(sql, "workfl_proc", "id");
                //return (id > 0);
                //PR 3.11.2017 
                try
                {
                    return WorkflowProces.KrokProcesu(str.id_poz, str.stav, 0, str.duvod, str.adresat, str.resitel, str.zapsal, str.datum, str.cas, sendmail);
                }
                catch (SmtpException)
                { return true; }
            }
            else
            {
                string sql = "update workfl_proc set "
                    + " resitel=" + SqlPrikaz.ValueToString(str.resitel) + ","
                    + " datum=" + SqlPrikaz.ValueToString(str.datum) + ","
                    + " cas=" + SqlPrikaz.ValueToString(s_cas)
                    + " where id_poz=" + SqlPrikaz.ValueToString(str.id_poz)
                    + " and stav=" + SqlPrikaz.ValueToString(str.stav);
                SqlPrikaz.ExecuteNonQuery(sql);
                return false;
            }
        }
    }

    //public class SCResult
    //{
    //    public int ErrorCode;
    //    public string ErrorString;
    //}

    //public class SCOsoba
    //{
    //    public decimal? OsCislo;
    //    public string Login;
    //}

    //public class SCZaloha
    //{
    //    public decimal Castka;
    //    [XmlElementAttribute(IsNullable = true)]
    //    public string KodMeny;
    //    [XmlElementAttribute(IsNullable = true)]
    //    public string Druh;
    //    public SCOsoba Pozadal;
    //    public DateTime? PozadalDne;
    //    public SCOsoba Schvalil;
    //    public DateTime? SchvalilDne;
    //    public SCOsoba Vyplatil;
    //    public DateTime? VyplatilDne;
    //    [XmlElementAttribute(IsNullable = true)]
    //    public string VyplJak;
    //    [XmlElementAttribute(IsNullable = true)]
    //    public string VyplDoklad;
    //}

    /// <summary>
    /// Průběh služební cesty (verze 2)
    /// </summary>
    public class SCPrubeh2
    {
        /// <summary>
        /// Událost - char(1){O-Odjezd, P-Příjezd, H-Přechod hranic, V-Zač. prac. výkonu, K-Konec prac. výkonu, R-Zač. přerušení, U-Konec přerušení}
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public string Udalost;
        /// <summary>
        /// Datum a čas
        /// </summary>
        public DateTime Datum;
        /// <summary>
        /// Místo 
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public string Misto;
        /// <summary>
        /// "Země" pro událost O a P, "Země ze které se přechází" pro událost H-Přechod hranic (nebo pro Odjezd letalem)
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public string ZemeOd;
        /// <summary>
        /// "Země do které se přechází" pro událost H-Přechod hranic (nebo pro Odjezd letalem)
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public string ZemeDo;
        /// <summary>
        /// Druh dopravy AUV, AUS, L, ... z číselníku druhy doprav. prostředků
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public string DruhDopr;
        /// <summary>
        /// Ujeté km "město" - jen pro SK legislativu
        /// </summary>
        public decimal? KmMesto;
        /// <summary>
        /// Ujeté km "mimo město" - jen pro SK legislativu
        /// </summary>
        public decimal? KmMimo;
        /// <summary>
        /// Ujeté km "kombinovaný režim" - jen pro SK legislativu
        /// </summary>
        public decimal? KmKomb;
        /// <summary>
        /// Ujeté km - pouze pro Udalost={H,P} (u události O je logicky 0)
        /// </summary>
        public decimal? KmCelkem;
    }

    /// <summary>
    /// Výdaj
    /// </summary>
    public class SCVydaj2
    {
        /// <summary>
        /// Datum uskutečnění výdaje
        /// </summary>
        public DateTime Datum;
        /// <summary>
        /// Částka
        /// </summary>
        public decimal Castka;
        /// <summary>
        /// Kód měny CZK, EUR, ...
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public string KodMeny;
        /// <summary>
        /// Druh výdaje (kód z číselníku druhy cestovních náhrad)
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public string Druh;
        /// <summary>
        /// Způsob platby (kód z číselníku způsob platby)
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public string ZpusPlat;
        /// <summary>
        /// Poznámka(účel) char(45)
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public string Pozn;
        /// <summary>
        /// Kód DPH z číselníku Sazby DPH
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public string KodDPH;
    }

    //public class SCWorkflowProc
    //{
    //    [XmlElementAttribute(IsNullable = true)]
    //    public string Stav;
    //    public DateTime Datum;
    //    [XmlElementAttribute(IsNullable = true)]
    //    public SCOsoba Resitel;
    //    [XmlElementAttribute(IsNullable = true)]
    //    public string Duvod;
    //}

    /// <summary>
    /// Cesta - údaje z hlavičky + vnořené struktury pro zálohy, průběh, výdaje a proces řešení
    /// </summary>
    public class SCCesta2
    {
        /// <summary>
        /// Číslo služební cesty char(25), pokud není zadáno vygeneruje systém
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public string Cislo;
        /// <summary>
        /// Zaměstnanec
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public SCOsoba Zamestnanec;
        /// <summary>
        /// Typ cesty char(1): t=tuzemská, z=zahraniční
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public string Typ;
        /// <summary>
        /// Místo počátku SC, pokud není zadáno vygeneruje systém
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public string PocMisto;
        /// <summary>
        /// Datum a čas počátku SC
        /// </summary>
        public DateTime DatOd;
        /// <summary>
        /// Místo konce SC, pokud není zadáno vygeneruje systém
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public string KonMisto;
        /// <summary>
        /// Datum a čas konce SC
        /// </summary>
        public DateTime DatDo;
        /// <summary>
        /// Cíl cesty(místo jednání) char(80)
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public string Cil;
        /// <summary>
        /// Cílová země - název dle číselníku zahraniční stravné
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public string CilZeme;
        /// <summary>
        /// Účel cesty char(80)
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public string Ucel;
        /// <summary>
        /// Druh nákladu - kód dle univerzálního číselníku „služební cesty-druh nákladu“ (např ŠKOLENÍ)
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public string DruhNakl;
        /// <summary>
        /// Druh dopravy, i více hodnot oddělených čárkou, AUV,AUS,L z číselníku Druhy doprav. prostředků
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public string DruhDopr;
        /// <summary>
        /// SPZ vozidla při AUV nebo AUS (nepovinné)
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public string Spz;
        /// <summary>
        /// Poznámka char(120)
        /// </summary>
        [XmlElementAttribute(IsNullable = true)]
        public string Poznamka;
        /// <summary>
        /// Kdo vytvořil SC
        /// </summary>
        public SCOsoba Vytvoril;
        public DateTime? VytvorDne;
        /// <summary>
        /// Datum souhlasu zaměstnance(cestujícího) se SC
        /// </summary>
        public DateTime? DatumSouhlasu;
        /// <summary>
        /// Kdo povolil SC
        /// </summary>
        public SCOsoba Povolil;
        public DateTime? PovolilDne;
        /// <summary>
        /// Seznam spolucestujících
        /// </summary>
        public List<SCOsoba> Spolucestujici;
        /// <summary>
        /// Seznam záloh
        /// </summary>
        public List<SCZaloha> Zalohy;
        /// <summary>
        /// Seznam záznamů pro Průběh (Odjezdy, příjezdy, přechody hranic)
        /// </summary>
        public List<SCPrubeh2> Prubeh;
        /// <summary>
        /// Seznam výdajů
        /// </summary>
        public List<SCVydaj2> Vydaje;
        /// <summary>
        /// Seznam proběhlých workflow stavů
        /// </summary>
        public List<SCWorkflowProc> WorkFlow;

        //predpripravene polozky ktere se ignoruji:
        //muzu je pridat i bez zmeny WS ci metody
        public Decimal? PredpCelkNakl;
        [XmlElementAttribute(IsNullable = true)]
        public string PCNKodMeny;
        //[XmlElementAttribute(IsNullable = true)]
        //public string UbytAno;//0 - ne, 1 - ano, 2 - žádám o zajištění
        //[XmlElementAttribute(IsNullable = true)]
        //public string UbytTxt;//kde        
        //public DateTime? PrerusOd;
        //public DateTime? PrerusDo;
        //[XmlElementAttribute(IsNullable = true)]
        //public string PrerusDuvod;        
        //[XmlElementAttribute(IsNullable = true)]
        //public string Firma;
        //----------------------------------------------
    }
}
