﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Services;
using KSProgram.Funkce;
using System.Data;
using System.Xml.Linq;
using System.Web.Script.Services;
using System.Xml.Serialization;
using System.Web.Services.Protocols;
using System.Xml;
using KSProgram.Data;
using KSProgram;

namespace KSPortal.WebServices.v_1_1
{
    /// <summary>
    /// Summary description for Zamestnanci
    /// </summary>
    /// Autor: Milan
    //[WebService(Namespace = "ksprtl/WebServices", Description = "Pokusná webová služba.")]
    [WebService(Namespace = "ksprtl/WebServices")]
    [WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
    [System.ComponentModel.ToolboxItem(false)]
    // To allow this Web Service to be called from script, using ASP.NET AJAX, uncomment the following line. 
    // [System.Web.Script.Services.ScriptService]
    public class Zamestnanci : System.Web.Services.WebService
    {
        public Auth Authentication;

        #region Metody webové služby
        [SoapHeader("Authentication")]
        //[WebMethod(EnableSession = true, Description = "Testovací metoda"), ScriptMethod(ResponseFormat = ResponseFormat.Xml)]
        [WebMethod(EnableSession = true, Description = "Metoda vrací informace o dovolené pro všechny aktivní zaměstnance."), ScriptMethod(ResponseFormat = ResponseFormat.Xml)]
        public ExportResultType VratPrehledDovolene()
        {
            ExportResultType result = new ExportResultType();

            try
            {
                // Autentizace
                Authenticate(Authentication);

                //PR 07.05.2021 HID=56482 nastavení databáze, aby to zařvalo chybou "Není nastavena výchozí databáze" nyní 
                //a ne "Při vykonávání SQL příkazu .. nastala chyba." v DejPrehledDovolene
                nastavDatabazi();

                // Získání informací o dovolené
                ZamestnanciType zam = DejPrehledDovolene();

                // Nastavení výsledku
                result.data.zamestnanci = zam;
                result.error.message = "";

            }
            catch (Exception ex)
            {
                KSProgram.ExceptionUtility.LogProcess(ex);//PR 07.05.2021 HID=56482 zalogování chyby, neboť skutečná chyba je často v InnerException
                result.error.message = ex.Message;
                return result;
            }

            return result;
        }
        #endregion

        #region Metody
        ZamestnanciType DejPrehledDovolene()
        {
            decimal os_cislo;
            short cislo_pom;
            DateTime? vystup;
            short vychoziPomer = 0;
            decimal predchozi_oscislo = -1;
            DateTime aktDate = DateTime.Today.Date;

            //Prac_pom pracPom = new Prac_pom(); // Instance objektu Prac_pom

            // Načte se seznam PPV platných k dnešnímu dni
            // DataTable pomery = Prac_pom.GetPomery("p", DateTime.Now.Date, "", false);
            //PH 17.10.2019 HID:46858
            ParametryPrikazu par = new ParametryPrikazu();
            DataTable pomery = Prac_pom.GetPomery("p", DateTime.Now.Date, "", false, par);

            // Seřazení podle os. čísla a čísla poměru 
            DataRow[] rows = pomery.Select(null, "os_cislo, cislo_pom");

            // Zpracují se údaje zaměstnance
            // -----------------------------
            ZamestnanciType zam = new ZamestnanciType();
            zam.zamestnanci = new List<ZamestnanecType>();

            ZamestnanecType zamestnanec = new ZamestnanecType();
            List<PPVType> PPVList = new List<PPVType>();


            foreach (DataRow row in rows)
            {
                os_cislo = Convert.ToDecimal(row["os_cislo"]);
                cislo_pom = Convert.ToInt16(row["cislo_pom"]);

                vystup = null;
                if (row["vystup"] != DBNull.Value)
                {
                    vystup = Convert.ToDateTime(row["vystup"]);
                }

                //if (os_cislo == 5) { break; }

                if (predchozi_oscislo != -1 && predchozi_oscislo != os_cislo)
                {
                    zam.zamestnanci.Add(zamestnanec);
                }

                if (os_cislo != predchozi_oscislo) // Nový zaměstnanec
                {
                    predchozi_oscislo = os_cislo;

                    zamestnanec = new ZamestnanecType();
                    zamestnanec.os_cislo = os_cislo;

                    PPVList = new List<PPVType>();

                    // Zjištění výchozího PPV
                    DateTime date = aktDate;
                    if (vystup != null)
                    {
                        if (vystup < aktDate) date = (DateTime)vystup;
                    }
                    Prac_pom.Platny_Pomer_Den(os_cislo, date, out vychoziPomer);
                }

                PPVType PPV = new PPVType();
                PPV.cisloPomeru = cislo_pom;
                PPV.jeVychoziPomer = (vychoziPomer == cislo_pom);
                PPV.dovolena = new DovolenaType(os_cislo, cislo_pom);

                PPVList.Add(PPV);
                zamestnanec.pracovniPomery = PPVList;
            }

            if (predchozi_oscislo != -1)
            {
                zam.zamestnanci.Add(zamestnanec);
            }

            return zam;
        }

        private void nastavDatabazi()
        {
            //PR 07.05.2021 pro testovací účely: možnost určit databázi přes queryString
            if (Globals.IsDevelopMode)
            {
                if (!string.IsNullOrEmpty(Context.Request.QueryString["dbname"]))
                    Globals.Db = DatabasePodleParam.Create(Context.Request.QueryString["dbname"]);
            }
            
            if (!Globals.IsDbConected)
                Globals.Db = DatabasePodleParam.Create(null);
        }
        #endregion

        #region Autentizace
        private void Authenticate(Auth aAuth)
        {
            if (aAuth == null)
            {
                aAuth = fakeAuthFromQueryString();//PR 07.05.2021 
                if (aAuth == null)
                    throw new Exception("Nelze provést autentizaci z důvodu chybějících autentizačních údajů!");
            }

            if (aAuth.Username == "pis" && aAuth.Password == "ksprg153")
            {
                return;
            }
            else
            {
                throw new Exception("Autentizace se nezdařila z důvodu uvedení chybného jména nebo hesla!");
            }
        }

        /// <summary>
        /// PR 07.05.2021 možnost zadat login přes queryString pro otestování WS přes prohlížeč.
        /// </summary>
        /// <returns></returns>
        private Auth fakeAuthFromQueryString()
        {
            try
            {
                
                if (!string.IsNullOrEmpty(Context.Request.QueryString["username"]))
                {
                    return new Auth()
                    {
                        Username = Context.Request.QueryString["username"],
                        Password = Context.Request.QueryString["password"]
                    };
                }
            }
            catch { }
            return null;
        }

        public class Auth : SoapHeader
        {
            public string Username;
            public string Password;
        }
        #endregion

    }

    #region Datové typy
    public class PPVType
    {
        [XmlAttribute(AttributeName = "cisloPomeru")]
        public short cisloPomeru;
        [XmlAttribute(AttributeName = "jeVychoziPomer")]
        public bool jeVychoziPomer;
        [XmlElement(ElementName = "DovolenaType")]
        public DovolenaType dovolena;
    }

    public class ZamestnanecType
    {
        [XmlAttribute(AttributeName = "osobniCislo")]
        public decimal os_cislo;
        [XmlArray(ElementName = "pracovniPomeryList")]
        public List<PPVType> pracovniPomery;
    }

    //[XmlType(TypeName="ZamestnanciType")]
    public class ZamestnanciType
    {
        [XmlArray(ElementName = "zamestnanciList")]
        public List<ZamestnanecType> zamestnanci;
    }
    //[XmlType(TypeName = "dovolena")]
    public class DovolenaType
    {
        /// <summary>
        /// Zákonný (fixní) nárok za letošní rok.
        /// </summary>
        [XmlAttribute(AttributeName = "narokZakonny")]
        public decimal NarokZakonny;
        /// <summary>
        /// Nárok na dovolenou za letoční rok.
        /// </summary>
        [XmlAttribute(AttributeName = "narokLetos")]
        public decimal NarokLetos;
        /// <summary>
        /// Zbývající nárok z minulého roku
        /// </summary>
        [XmlAttribute(AttributeName = "narokMin")]
        public decimal NarokMin;
        /// <summary>
        /// Nárok na dovolenou celkem.
        /// </summary>
        [XmlAttribute(AttributeName = "narokCelkem")]
        public decimal NarokCelkem;
        /// <summary>
        /// Čerpaná dovolená k počátku 'Obdobi'.
        /// </summary>
        [XmlAttribute(AttributeName = "cerpana")]
        public decimal Cerpana;
        /// <summary>
        /// Čerpaná dovolená v 'Obdobi'.
        /// </summary>
        [XmlAttribute(AttributeName = "cerpanaAktualne")]
        public decimal CerpanaAktualne;
        /// <summary>
        /// Čerpaná dovolená celkem.
        /// </summary>
        [XmlAttribute(AttributeName = "cerpanaCelkem")]
        public decimal CerpanaCelkem;
        /// <summary>
        /// Krácená dovolená
        /// </summary>
        [XmlAttribute(AttributeName = "kracena")]
        public decimal Kracena;
        /// <summary>
        /// Dodatková dovolená
        /// </summary>
        [XmlAttribute(AttributeName = "dodatkova")]
        public decimal Dodatkova;
        /// <summary>
        /// Celková zbývající dovolená.
        /// </summary>
        [XmlAttribute(AttributeName = "zbyvajici")]
        public decimal Zbyvajici;
        /// <summary>
        /// Nevyčerpaná dovolená z minulých let.
        /// </summary>
        [XmlAttribute(AttributeName = "zbyvajiciZMinObdobi")]
        public decimal ZbyvajiciZMinObdobi;

        #region Konstruktory
        /// <summary>
        /// Výchozí konstruktor
        /// </summary>
        public DovolenaType()
        {
        }
        /// <summary>
        /// Konstruktor s parametry pro nastavení atributů.
        /// </summary>
        /// <param name="os_cislo"></param>
        /// <param name="cislo_pom"></param>
        public DovolenaType(decimal os_cislo, short cislo_pom)
        {
            // Informace o dovolené
            // --------------------
            KSProgram.Funkce.Dovolena dovolena = new KSProgram.Funkce.Dovolena(os_cislo, cislo_pom);
            StrDovolena strDovInfo = dovolena.DovInfo();

            Cerpana = strDovInfo.Cerpana;
            CerpanaAktualne = strDovInfo.CerpanaAktualne;
            CerpanaCelkem = strDovInfo.CerpanaCelkem;
            Dodatkova = strDovInfo.Dodatkova;
            Kracena = strDovInfo.Kracena;
            NarokCelkem = strDovInfo.NarokCelkem;
            NarokLetos = strDovInfo.NarokLetos;
            NarokMin = strDovInfo.NarokMin;
            NarokZakonny = strDovInfo.NarokZakonny;
            Zbyvajici = strDovInfo.Zbyvajici;
            ZbyvajiciZMinObdobi = strDovInfo.ZbyvajiciZMinObdobi;
        }
        #endregion
    }

    public class ExportResultType
    {
        [XmlElement(ElementName = "DataType")]
        public DataType data;
        [XmlElement(ElementName = "ErrorType")]
        public ErrorType error;

        public ExportResultType()
        {
            data = new DataType();
            error = new ErrorType();
        }
    }

    public class ErrorType
    {
        public string message;
    }

    public class DataType
    {
        public ZamestnanciType zamestnanci;
    }
    #endregion


}
